/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.common;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 *
 * @author tskoda
 */
public class CommandLineParser {

    public static final String PROPERTY_DELIMITER = "=";
    public static final String PROPERTY_PREFIX = "--";
    public static final String QUOTE = "\"";
    
    private CommandLineParser() {
    }

    /**
     * Reads properties from array of Strings. Elements are expected to be in form
     * key=value or --key value.
     * @param args array of key and property pairs
     * @throws IllegalArgumentException when pair is incomplete
     */
    public static Map<String, String> toProperties(String[] args) {
        HashMap<String, String> result = new HashMap<String, String>();
        for (int i = 0; i < args.length; i++) {

            String[] pair = null;

            if ((args[i].startsWith(PROPERTY_PREFIX) && (i + 1) < args.length)) {
                pair = new String[2];
                pair[0] = args[i].substring(2);
                pair[1] = args[++i];
            } else if (args[i].contains(PROPERTY_DELIMITER)) {
                pair = args[i].split(PROPERTY_DELIMITER, 2);
            }

            if ((pair == null) || (pair.length != 2)) {
                throw new IllegalArgumentException("Either parameter name or its value is missing: " + args[i]);
            }
            
            //remove double quotes if present, better to use " on windows and escape store folder on linux/mac
            if ((pair[1].startsWith(QUOTE)) && (pair[1].length() > 1) && (pair[1].endsWith(QUOTE))) {
                pair[1] = pair[1].substring(1, pair[1].length() - 1);
            }
//            char type = DataTypes.decodeTypeIdFromName(pair[0]);
//            String propertyName = DataTypes.stripNameFromTypeId(pair[0]);
//            Object propertyType = DataTypes.decodeValueType(type, pair[1]);

            result.put(pair[0], pair[1]);
        }
        
        return result;
    }

    /**
     * Saves content of properties into String array. Properties defined in extraFormat parameter
     * are saved as --{key} {value} (two elements in result array), the rest of properties
     * are saved as {key}={value} (one element in result array). 
     * @param extraFormatProperty
     * @return 
     */
    public static String[] toArray(Map<String, String> properties, String... extraFormatProperty) {
        List<Object> result = new ArrayList();
        Set<Object> extra = new HashSet<Object>(Arrays.asList(extraFormatProperty));
        for (Map.Entry<String, String> pair : properties.entrySet()) {
            if (extra.contains(pair.getKey())) {
                result.add(PROPERTY_PREFIX + pair.getKey());
                result.add(pair.getValue());
            } else {
                result.add(pair.getKey() + PROPERTY_DELIMITER + pair.getValue());
            }
        }

        return result.toArray(new String[result.size()]);
    }
}
