/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.common;

import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 *
 * @author tskoda
 */
public final class InetAddressUtil {
    
    private InetAddressUtil() {
    }
    
    /**
     * Determine if provided string is valid IPv4 address.
     * @param inetAddress
     * @return true only if inetAddress is valid IPv4 address, false otherwise
     */
    public static boolean isIPv4(String inetAddress) {
        boolean result;
        try {
            result = (InetAddress.getByName(inetAddress) instanceof Inet4Address);
        } catch (UnknownHostException e) {
            result = false;
        }
        
        return result;
    }
    
    public static byte[] toBytes(InetAddress inetAddress) {
        return inetAddress.getAddress();
    }
    
//    /**
//     * Converts both ipv4 or ipv6 address to long.
//     * @return 
//     */
//    public static long toLong(InetAddress inetAddress) {
//        return toLong(inetAddress.getAddress());
//    }
//    
//    /**
//     * Converts 4byte or 8byte array to long, otherwise return -1.
//    
//     * @param array
//     * @return 
//     */
//    public static long toLong(byte[] array) {
//       long result = -1;
//       
//       ByteBuffer buffer = ByteBuffer.wrap(array);
//       
//       if (array.length <= 4) {
//           result = Long.valueOf(buffer.getInt());
//       } else if (array.length <= 8) {
//           result = buffer.getLong();
//       }
//       
//       return result;
//    }
    
    public static String combineURL(String prefix, String suffix) {
        
        if (prefix == null || suffix == null) {
            return "";
        }
        
        int lastPrefixIndex = prefix.length();
        int firstSuffixIndex = 0;
        
        while (lastPrefixIndex > 0 && prefix.charAt(lastPrefixIndex - 1) == '/') {
            lastPrefixIndex--;
        }
        
        while (firstSuffixIndex < suffix.length() && suffix.charAt(firstSuffixIndex) == '/') {
            firstSuffixIndex++;
        }
        
        return prefix.substring(0, lastPrefixIndex) + '/' + suffix.substring(firstSuffixIndex);
    }
  
}
