/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.common;

import java.io.File;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 *
 * @author tskoda
 */
public class TypedProperties {

    private static final String ONE = "1";
    private static final String TRUE = "true";
    private static final String YES = "yes";
    
    public static Long loong(Key<Long> key, Map<String, String> properties) {
        Long result = loong(key, properties, null);
        if (result == null) {
            throw new IllegalArgumentException("Key " + key + " not found");
        }

        return result;
    }

    public static Long loong(Key<Long> key, Map<String, String> properties, Long defaultValue) {
        Long result = null;
        if (key != null) {
            if ((properties != null) && (properties.containsKey(key.key))) {
                try {
                    result = Long.valueOf(properties.get(key.key));
                } catch (NumberFormatException e) {
                    throw new IllegalArgumentException("Value for key " + key + " can not be cast to integer");
                }
            } else {
                result = defaultValue;
            }
        }

        return result;
    }

    public static Integer integer(Key<Integer> key, Map<String, String> properties) {
        Integer result = integer(key, properties, null);
        if (result == null) {
            throw new IllegalArgumentException("Key " + key + " not found");
        }

        return result;
    }

    public static Integer integer(Key<Integer> key, Map<String, String> properties, Integer defaultValue) {
        Integer result = null;
        if (key != null) {
            if ((properties != null) && (properties.containsKey(key.key))) {
                try {
                    result = Integer.valueOf(properties.get(key.key));
                } catch (NumberFormatException e) {
                    throw new IllegalArgumentException("Value for key " + key + " can not be cast to integer");
                }
            } else {
                result = defaultValue;
            }
        }

        return result;
    }

    public static Boolean bool(Key<Boolean> key, Map<String, String> properties) {
        Boolean result = bool(key, properties, null);
        if (result == null) {
            throw new IllegalArgumentException("Key " + key + " not found");
        }

        return result;
    }

    public static Boolean bool(Key<Boolean> key, Map<String, String> properties, Boolean defaultValue) {
        Boolean result = null;
        if (key != null) {
            if ((properties != null) && (properties.containsKey(key.key))) {
                String property = properties.get(key.key);
                if ((ONE.equals(property)) || (TRUE.equalsIgnoreCase(property)) || (YES.equalsIgnoreCase(property))) {
                    result = Boolean.TRUE;
                } else {
                    result = Boolean.FALSE;
                }
            } else {
                result = defaultValue;
            }
        }

        return result;
    }

    public static File file(Key<File> key, Map<String, String> properties) {
        File result = file(key, properties, null);

        if (result == null) {
            throw new IllegalArgumentException("Key " + key + " not found");
        }

        return result;
    }

    public static File file(Key<File> key, Map<String, String> properties, File defaultFile) {
        File result = null;
        if (key != null) {
            if ((properties != null) && (properties.containsKey(key.key))) {
                result = new File(properties.get(key.key));
            } else {
                return defaultFile;
            }
        }

        return result;
    }

    public static String string(Key<String> key, Map<String, String> properties) {
        String result = string(key, properties, null);

        if (result == null) {
            throw new IllegalArgumentException("Key " + key + " not found");
        }

        return result;
    }

    public static String string(Key<String> key, Map<String, String> properties, String defaultValue) {
        String result = null;

        if (key != null) {
            if ((properties != null) && (properties.containsKey(key.key))) {
                result = properties.get(key.key);
            } else {
                result = defaultValue;
            }
        }

        return result;
    }

    public static TypedProperties asCopy(TypedProperties from) {
        TypedProperties result = new TypedProperties();
        result.properties.putAll(from.properties);
        return result;
    }
    protected final HashMap<Key, Object> properties;

    public TypedProperties() {
        this.properties = new HashMap<Key, Object>();
    }

    public void putAll(TypedProperties from) {
        if (from != null) {
            this.properties.putAll(from.properties);
        }
    }

    public <T> void put(Key<T> key, T value) {
        this.properties.put(key, value);
    }

    public <T> T get(Key<T> key) {
        return (T) this.properties.get(key);
    }

    public <T> T get(Key<T> key, T defaultValue) {
        T result;
        if (this.properties.containsKey(key)) {
            result = (T) this.properties.get(key);
        } else {
            result = defaultValue;
        }
        return result;
    }
}
