/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.api.service;

/**
 *
 * @author tskoda
 */
public interface AuthService {

    boolean auth(Credentials credentials, Connection connection);
    void disconnected(Credentials credentials);
    
    class Credentials {
        private final String username;
        private final String domain;
        private final String login;
        private final byte[] pass;

        public Credentials(String username, String domain, byte[] pass) {
            this.username = username.toLowerCase();
            this.domain = domain.toLowerCase();
            this.login = this.username + "@" + this.domain;
            this.pass = pass;
        }

        public String getUsername() {
            return username;
        }

        public String getDomain() {
            return domain;
        }
        
        public String getLogin() {
            return login;
        }

        public byte[] getPass() {
            return pass;
        }

    }
    
    class Connection {
        private final int localPort;
        private final boolean localPortSecure;
        private final String remoteIP;
        private final int remotePort;
        private final boolean tlsRequired;

        public Connection(int localPort, boolean localPortSecure, String remoteIP, int remotePort, boolean tlsRequired) {
            this.localPort = localPort;
            this.localPortSecure = localPortSecure;
            this.remoteIP = remoteIP;
            this.remotePort = remotePort;
            this.tlsRequired = tlsRequired;
        }

        public int getLocalPort() {
            return localPort;
        }

        public boolean isLocalPortSecure() {
            return localPortSecure;
        }

        public String getRemoteIP() {
            return remoteIP;
        }
        
        public int getRemotePort() {
            return remotePort;
        }

        public boolean isTlsRequired() {
            return tlsRequired;
        }
    }
}
