/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.api.service;

/**
 * Serves as a system wide bus.
 *
 * @author tskoda
 */
public interface ServiceBus {

    /**
     * Publish event to all listeners. All listeners are notified sequentional.
     * Method must return after all listeners have been notified.
     *
     * @param <T>
     * @param event
     */
    <T extends Event> void publish(T event);

    <T extends Event> void publish(T event, ExceptionHandler handler);

    <T extends Event> void subscribe(Class< T> event, EventListener< T> listener);

    interface Event {
    }

    interface EventListener<T extends Event> {

        void event(T event);
    }

    interface ExceptionHandler {

        boolean pass(Exception e);
    }
}
