/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core;

import com.kerio.im.common.error.Code;
import com.kerio.im.common.error.ServiceException;
import com.kerio.im.core.Shutdown;
import com.kerio.im.core.api.service.ErrorService;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class ErrorHandler implements ErrorService {

    private static final Logger LOG = Logger.getLogger(ErrorHandler.class.getName());
    private final Shutdown shutdown;

    public ErrorHandler(Shutdown shutdown) {
        this.shutdown = shutdown;
        Thread.setDefaultUncaughtExceptionHandler(new ThreadExceptionHandler());
    }

    @Override
    public void error(Code code) {
        this.shutdown.shutdown(code);
    }

    private class ThreadExceptionHandler implements Thread.UncaughtExceptionHandler {

        @Override
        public void uncaughtException(Thread t, Throwable e) {
            LOG.log(Level.INFO, "Caught exception from thread {0}: {1}", new Object[]{t.getName(), e});
            Code code = Code.INTERNAL_SERVER_ERROR;

            if (e instanceof ServiceException) {
                code = ((ServiceException) e).getCode();
            }

            ErrorHandler.this.error(code);
        }
    }
}
