/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.auth;

import com.kerio.im.common.error.Code;
import com.kerio.im.core.ErrorHandler;
import com.kerio.im.core.api.service.AuthService;
import com.kerio.im.core.api.service.AuthService.Credentials;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class OfflineAuthentication {

    private static final Logger LOG = Logger.getLogger(OfflineAuthentication.class.getName());
    private final String login;
    private final String domain;
    private final int port;
    private final ErrorHandler errorHandler;

    public OfflineAuthentication(ErrorHandler errorHandler, String login, String domain, int port) {
        this.login = login;
        this.domain = domain;
        this.port = port;
        this.errorHandler = errorHandler;
    }
    
    private boolean isServiceUser(Credentials credentials) {
        return this.login.equals(credentials.getUsername())
                && this.domain.equals(credentials.getDomain());
    }

    private boolean isServiceUser(Credentials credentials, int port) {
        return isServiceUser(credentials)
                && this.port == port;
    }

    public boolean auth(Credentials credentials, AuthService.Connection connection) {

        boolean result;
        if (result = isServiceUser(credentials, connection.getLocalPort())) {
            LOG.log(Level.INFO, "Authenticating control user {0} succeeded", login);
        } else {
            LOG.log(Level.INFO, "Authenticating control user {0} failed", login);
        }

        return result;
    }

    public void disconnected(Credentials credentials) {
        if (isServiceUser(credentials)) {
            LOG.log(Level.INFO, "Control user {0}@{1} disconnected, closing im service", new Object[]{this.login, this.domain});
            this.errorHandler.error(Code.CONTROL_DISCONNECTED);
        }
    }
}
