/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.auth;

import com.kerio.im.core.api.service.AuthService;
import com.kerio.im.core.service.ConnectionListener;
import com.kerio.im.core.service.ServiceManager;
import com.kerio.im.core.user.Users;
import com.kerio.im.external.ExtService;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class OnlineAuthentication implements AuthService {

    private static final Logger LOG = Logger.getLogger(OnlineAuthentication.class.getName());
    private final Users users;
    private final ExtService extService;
    private final ServiceManager ports;

    public OnlineAuthentication(final Users users, final ServiceManager ports, final ExtService extService) {
        this.users = users;
        this.ports = ports;
        this.extService = extService;
    }

    @Override
    public boolean auth(Credentials credentials, Connection connection) {

        String serviceId = this.ports.getServiceId(connection.getLocalPort());
        String connectionId = ConnectionListener.asConnectinId(connection.getRemoteIP(), connection.getRemotePort());
        boolean result = this.extService.auth(
                credentials.getLogin(),
                credentials.getPass(),
                serviceId,
                connectionId,
                connection.isLocalPortSecure());
        if (result) {
            if (result = this.users.check(credentials.getUsername(), credentials.getDomain())) {
                LOG.log(Level.INFO, "Authenticating user {0} succeeded", credentials.getLogin());
            } else {
                LOG.log(Level.INFO, "Authenticating user {0} failed, user is not synchronized yet", credentials.getLogin());
            }
        } else {
            LOG.log(Level.INFO, "Authenticating user {0} failed, incorrect credentials", credentials.getLogin());
        }
        return result;
    }

    @Override
    public void disconnected(Credentials credentials) {
        /* empty handler, disconnects are handler by ConnectionListener
        String serviceId = connectionToService.get(credentials.getConnectionId());

        if (serviceId != null) {
            try {
                this.extService.disconnect(serviceId, credentials.getConnectionId());
            } finally {
                this.connectionToService.remove(credentials.getConnectionId());
            }
        }
        */ 
    }
}
