/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.domain;

import com.kerio.im.bean.Domain;
import com.kerio.im.external.ExtService;
import com.kerio.im.xmpp.api.XmppDomains;
import com.kerio.im.xmpp.api.XmppException;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
//TODO renam to FullSync
public class DomainFullSync implements Runnable {

    private static final Logger LOG = Logger.getLogger(DomainFullSync.class.getName());
    private final XmppDomains xmppDomains;
    private final ExtService extService;
    private final DomainControl domainControl;

    public DomainFullSync(XmppDomains xmppDomains, ExtService extService, DomainControl domainControl) {
        this.xmppDomains = xmppDomains;
        this.extService = extService;
        this.domainControl = domainControl;
    }

    @Override
    public void run() {
        this.sync();
    }

    public void sync() {
        LOG.log(Level.FINE, "Synchronizing domains");
        synchronized (this.domainControl) {
            try {
                ExtDomainsHandler extDomainsHandler = new ExtDomainsHandler(xmppDomains.all(), this.domainControl);
                //TODO could block api for long time
                this.extService.getAllDomains(extDomainsHandler);
                int removed = 0;
                for (String domain : extDomainsHandler.notFound()) {
                    try {
                        if (DomainFullSync.this.domainControl.remove(domain)) {
                            removed++;
                        }
                    } catch (XmppException e) {
                        LOG.log(Level.WARNING, e.getMessage());
                    }
                }
                LOG.log(Level.INFO, "Synchronizing domains successful, {0} added, {1} removed, {2} checked, {3} skipped", new Object[]{extDomainsHandler.added, removed, extDomainsHandler.checked, extDomainsHandler.skipped});
            } catch (Exception e) {
                LOG.log(Level.INFO, "Synchronizing domains failed", e);
            }
        }
    }

    private static class ExtDomainsHandler implements ExtService.DomainHandler {
        
        private int checked = 0;
        private int added = 0;
        private int skipped = 0;
        private final Set<String> allDomains;
        private final DomainControl domainControl;

        public ExtDomainsHandler(Set<String> xmppDomains, DomainControl domainControl) {
            this.allDomains = new HashSet<String>(xmppDomains);
            this.domainControl = domainControl;
        }
        

        @Override
        public void handle(String id, String name) {
            
            try {
                String lowerCaseName = name.toLowerCase();
                if (allDomains.contains(lowerCaseName)) {
                    allDomains.remove(lowerCaseName);
                    if (this.domainControl.check(new Domain(id, lowerCaseName))) {
                        checked++;
                    }
                } else {
                    if (this.domainControl.add(new Domain(id, lowerCaseName))) {
                        added++;
                    }
                }
            } catch (XmppException e) {
                LOG.log(Level.WARNING, "Failed to process domain " + name, e);
                skipped++;
            }
        }
        
        public Set<String> notFound() {
            return this.allDomains;
        }
    }
}
