/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.domain;

import com.kerio.im.bean.Domain;
import com.kerio.im.core.ThreadPool;
import com.kerio.im.core.api.GlobalKey;
import com.kerio.im.core.api.event.ShutdownEvent;
import com.kerio.im.core.api.service.ConfigurationService;
import com.kerio.im.core.api.service.ServiceBus;
import com.kerio.im.core.cert.Certificates;
import com.kerio.im.core.remote.CommandEvent;
import com.kerio.im.external.ExtService;
import com.kerio.im.xmpp.api.XmppDomains;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class Domains {

    private static final Logger LOG = Logger.getLogger(Domains.class.getName());
    public static final int INITIAL_DELAY = 5;
    private final DomainControl domainControl;

    public Domains(final ConfigurationService configuration, final ServiceBus bus, final ThreadPool threads, final ExtService extService, final XmppDomains xmppDomains) {
        this.domainControl = new DomainControl(
                xmppDomains, 
                new Domain(
                    configuration.get(GlobalKey.SERVICE_REMOTE_DOMAIN),
                    configuration.get(GlobalKey.SERVICE_REMOTE_DOMAIN)),
                bus);
        
        LOG.log(Level.FINE, "Domain synchronization subscribed for change events");
        bus.subscribe(CommandEvent.class, new DomainEventSync(domainControl));   
        bus.subscribe(ShutdownEvent.class, threads.submit(
                new DomainFullSync(xmppDomains, extService, this.domainControl), 
                INITIAL_DELAY, 
                configuration.get(GlobalKey.REFRESH_PERIOD_KEY), 
                TimeUnit.SECONDS));
        LOG.log(Level.INFO, "Domain synchronization scheduled, it's about to start in {0} seconds", INITIAL_DELAY);
    }

    public Set<Domain> all() {
        synchronized(this.domainControl) {
            return this.domainControl.allExceptService();
        }
    }
    
    public boolean isLocalDomain(String domain) {
        synchronized(this.domainControl) {
            return this.domainControl.isLocalDomain(domain);
        }
    }
}
