/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.logging;

import com.kerio.im.common.TypedProperties;
import com.kerio.im.common.error.Code;
import com.kerio.im.core.api.GlobalKey;
import com.kerio.im.xmpp.api.XmppException;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Map;
import java.util.logging.Level;

/**
 *
 * @author tskoda
 */
public class LoggingConfiguration {

    public static final byte[] CONFIGURATION = (""
            + "handlers=com.kerio.im.core.logging.ConnectHandler, com.kerio.im.core.logging.FileHandler\n"
            //+ "tigase.net.IOService.handlers=com.kerio.im.core.logging.StanzaHandler\n"
            //+ "tigase.net.IOService.useParentHandlers=false\n"
            //+ "tigase.net.IOService.level=FINEST\n"
            + ""
            + ".level=FINEST\n"
            + ""
            + "sun.net.www.protocol.http.HttpURLConnection.level=INFO\n"
            + "com.kerio.lib.json.api.client.KJsonRpcClient.level=INFO\n" //passwords could be logged!
            + "org.jabsorb.level=WARNING\n"
            + "sun.net.www=WARNING\n"
            + "tigase.xml.level=INFO\n"
            + "tigase.io.level=INFO\n"
            + "org.eclipse.jetty.level=INFO\n"
            //+ "tigase.xmpp.impl.level=INFO\n"
            + "tigase.server.xmppsession.SessionManager.level=INFO\n"
            + "tigase.xmpp.impl.Presence.level=INFO\n"
            + "tigase.util.DNSResolver.level=SEVERE\n"
            + "com.kerio.im.tigase.plugin.IMBridgePlugin=INFO\n"
            + "javax.management.level=SEVERE\n"
            + "com.kerio.im.tigase.bridge.level=FINE\n"
            + "tigase.muc.level=FINER\n").getBytes();
    private final TypedProperties properties;
    private boolean stdRedirected;

    protected LoggingConfiguration(TypedProperties properties) {
        this.properties = properties;
        defaults();
    }

    private void defaults() {
        this.properties.put(LogKeys.DEBUG, Boolean.FALSE);
        this.properties.put(LogKeys.ENGINE_LEVEL, Level.FINER);
        this.properties.put(LogKeys.ENGINE_SIMPLE, Boolean.TRUE);
        this.properties.put(LogKeys.ENGINE_OUTPUT, LoggingManager.STDERR_STREAM);
        this.properties.put(LogKeys.FILE_LEVEL, Level.FINER);
        this.properties.put(LogKeys.STANZA_ENABLED, Boolean.TRUE);
        this.properties.put(LogKeys.STANZA_LEVEL, Level.INFO);

        stdRedirected = false;
    }

    public TypedProperties getProperties() {
        return properties;
    }

    public void configure(Map<String, String> configuration) {
        if (configuration.containsKey(LogKeys.DEBUG.key)) {
            this.properties.put(LogKeys.DEBUG, TypedProperties.bool(LogKeys.DEBUG, configuration));
        }

        if (configuration.containsKey(LogKeys.ENGINE_SIMPLE.key)) {
            this.properties.put(LogKeys.ENGINE_SIMPLE, TypedProperties.bool(LogKeys.ENGINE_SIMPLE, configuration));
        }

        if (configuration.containsKey(LogKeys.STANZA_ENABLED.key)) {
            this.properties.put(LogKeys.STANZA_ENABLED, TypedProperties.bool(LogKeys.STANZA_ENABLED, configuration));
        }

        if (configuration.containsKey(LogKeys.ENGINE_LEVEL.key)) {
            this.properties.put(LogKeys.ENGINE_LEVEL, Level.parse(configuration.get(LogKeys.ENGINE_LEVEL.key)));
        }

        if (configuration.containsKey(GlobalKey.DATA_DIR.key)) {
            File data = TypedProperties.file(GlobalKey.DATA_DIR, configuration);
            properties.put(LogKeys.FILE, new File(data, "logs/im.log"));
            properties.put(LogKeys.STANZA, new File(data, "logs/stanza.log"));
        }

        if (configuration.containsKey(LogKeys.ENGINE_PIPE.key)) {
            this.properties.put(LogKeys.ENGINE_OUTPUT, openPipe(configuration.get(LogKeys.ENGINE_PIPE.key)));
        }
    }

    private OutputStream openPipe(String name) {
        OutputStream result = null;

        if (name != null && name.startsWith("\\\\.\\pipe\\")) {
            try {
                result = new FileOutputStream(name);
            } catch (IOException e) {
                throw new XmppException("Failed to open pipe {0}", e, Code.INTERNAL_SERVER_ERROR, name);
            }
        } else {
            throw new XmppException("Failed to open pipe {0}", Code.INTERNAL_SERVER_ERROR, name);
        }
        
        return result;
    }

    public void stdRedirected() {
        this.stdRedirected = true;
    }

    public boolean isStdRedirected() {
        return this.stdRedirected;
    }
}
