/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.service;

import com.kerio.im.core.api.service.ServiceBus;
import com.kerio.im.core.remote.CommandEvent;
import com.kerio.im.external.ExtService;
import com.kerio.im.xmpp.api.XmppPorts;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class EventFederation implements ServiceBus.EventListener<CommandEvent> {

    private static final Logger LOG = Logger.getLogger(EventFederation.class.getName());
    private static final String MESSAGE = "settings";
    private static final int FEDERATION_SERVICE_PORT = 5269;
    private static final String FEDERATION_SERVICE_NAME = "Instant Messaging Federation";
    private static final String FEDERATION_SERVICE_ANY_ADDRESS = "*"; //all addresses
    public static final String FEDERATION_SERVICE_ID = "//federation/" + FEDERATION_SERVICE_ANY_ADDRESS + ":" + FEDERATION_SERVICE_PORT;
    private static final boolean PLAIN = false;
    private final ServicePort federationPort;
    private final ServiceProcessor serviceProcessor;
    private boolean sendOutsideEnabled;

    public EventFederation(ExtService extService, ServiceProcessor serviceProcessor, String hostname) {
        this.federationPort = new ServicePort(
                FEDERATION_SERVICE_ID,
                FEDERATION_SERVICE_PORT,
                FEDERATION_SERVICE_ANY_ADDRESS,
                hostname,
                PLAIN,
                ServiceManager.SERVICE_SECURITY_NOT_REQUIRED,
                FEDERATION_SERVICE_NAME,
                CombinedFilter.without(), //no ip filtering
                ConnectionLimit.without(), //no connection limit
                ConnectionListener.deaf()); //we do not notify inter-server connections
        this.serviceProcessor = serviceProcessor;
        this.sendOutsideEnabled = extService.getSettings().isSendOutsideEnabled();

        if (this.sendOutsideEnabled) {
            try {
                startService();
            } catch (Exception e) {
                 LOG.log(Level.INFO, "Starting federation service failed", e);
            }
        }
    }

    private void startService() {
        LOG.log(Level.FINE, "Starting service {0}", FEDERATION_SERVICE_ID);
        this.serviceProcessor.start(FEDERATION_SERVICE_ID, ServiceProcessor.asSet(federationPort), XmppPorts.Type.SERVER);
        LOG.log(Level.INFO, "Starting service {0} successful", FEDERATION_SERVICE_ID);
    }

    @Override
    public void event(CommandEvent event) {
        if (!event.interested(MESSAGE)) {
            return;
        }

        try {
            boolean newSendOutsideEnabled = Boolean.valueOf(event.get("sendOutsideEnabled"));

            if (newSendOutsideEnabled != this.sendOutsideEnabled) {
                if (newSendOutsideEnabled) {
                    startService();
                } else {
                    LOG.log(Level.FINE, "Stopping service {0}", FEDERATION_SERVICE_ID);
                    this.serviceProcessor.stop(FEDERATION_SERVICE_ID, XmppPorts.Type.SERVER);
                    LOG.log(Level.FINE, "Stopping service {0} successful", FEDERATION_SERVICE_ID);
                }
                this.sendOutsideEnabled = newSendOutsideEnabled;
            }
        } catch (Exception e) {
            LOG.log(Level.WARNING, "Processing federation settings event failed", e);
            event.fail(e.getMessage());
        }
    }
}
