/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.service;

import com.kerio.im.bean.Port;
import java.math.BigInteger;
import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;

/**
 *
 * @author tskoda
 */
public class FilterMask implements Port.Filter {

    public static final FilterMask create(String netOrPrefix, String maskOrLength) throws UnknownHostException {
        try {
            //CIDR length
            return new FilterMask(netOrPrefix, Integer.parseInt(maskOrLength));
        } catch (NumberFormatException e) {
            //IP mask
            return new FilterMask(netOrPrefix, maskOrLength);
        }
    }

    private final String info;
    private final BigInteger net;
    private final BigInteger mask;
    private final BigInteger subnet;

    private FilterMask(String net, String mask) throws UnknownHostException {
        this.info = "FilterMask{" + net + "/" + mask + '}';
        
        this.net = new BigInteger(1, InetAddress.getByName(net).getAddress());
        this.mask = new BigInteger(1, InetAddress.getByName(mask).getAddress());
        this.subnet = this.net.and(this.mask);

    }

    private FilterMask(String net, int length) throws UnknownHostException {
        this.info = "FilterMask{" + net + "/" + length + '}';
        
        InetAddress netAddress = InetAddress.getByName(net);
        ByteBuffer buffer = getBufferForAddress(netAddress); 
        
        this.net = new BigInteger(1, netAddress.getAddress());
        this.mask = (new BigInteger(1, buffer.array())).not().shiftRight(length);
        this.subnet = this.net.and(this.mask);
    }

    private ByteBuffer getBufferForAddress(InetAddress address) {
        if (address instanceof Inet4Address) {
            return ByteBuffer.allocate(4).putInt(-1);
        } else {
            return ByteBuffer.allocate(16).putLong(-1L).putLong(-1L);
        }
    }

    @Override
    public boolean pass(InetAddress client) {
        BigInteger clientNumber = new BigInteger(1, client.getAddress());
        return clientNumber.and(this.mask).equals(this.subnet);
    }

    @Override
    public String toString() {
        return this.info;
    }

}
