/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.service;

import com.kerio.im.bean.Port;
import java.math.BigInteger;
import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 *
 * @author tskoda
 */
public class FilterRange implements Port.Filter {
    private final BigInteger minimum;
    private final BigInteger maximum;
    private final String info;

    public FilterRange(String from, String to) throws UnknownHostException {
        this(InetAddress.getByName(from), InetAddress.getByName(to));
    }
    
    public FilterRange (InetAddress fromAdd, InetAddress toAdd) {
        this.minimum = new BigInteger(1, fromAdd.getAddress());
        this.maximum = new BigInteger(1, toAdd.getAddress());
        this.info = "FilterRange{" + "from=" + fromAdd + ", to=" + toAdd + '}';
    }
    
    FilterRange (BigInteger minimum, BigInteger maximum) {
        this.minimum = minimum;
        this.maximum = maximum;
        this.info = "FilterRange{" + "from(int)=" + minimum + ", to(int)=" + maximum + '}';
    }

    @Override
    public boolean pass(InetAddress client) {
        BigInteger current = new BigInteger(1, client.getAddress());
        int lr = minimum.compareTo(current);
        int hr = maximum.compareTo(current);
        
        return (lr <= 0) && (hr >= 0);
    }
    
    @Override
    public String toString() {
        return info;
    }

}
