package com.kerio.im.core.auth;

import static org.junit.Assert.assertEquals;
import static org.mockito.AdditionalMatchers.or;
import static org.mockito.Matchers.anyBoolean;
import static org.mockito.Matchers.isA;
import static org.mockito.Matchers.isNull;

import com.diffblue.deeptestutils.Reflector;
import com.diffblue.deeptestutils.mock.DTUMemberMatcher;
import com.kerio.im.bean.Domain;
import com.kerio.im.core.api.service.AuthService.Connection;
import com.kerio.im.core.api.service.AuthService.Credentials;
import com.kerio.im.core.service.ServiceManager;
import com.kerio.im.core.service.ServiceProcessor;
import com.kerio.im.core.user.GroupProvider;
import com.kerio.im.core.user.UserControl;
import com.kerio.im.core.user.UserControlList;
import com.kerio.im.core.user.Users;
import com.kerio.im.external.ExtService;
import com.kerio.im.xmpp.api.XmppPorts;
import com.kerio.im.xmpp.api.XmppUsers;
import java.lang.reflect.Method;
import java.util.HashMap;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.junit.rules.Timeout;
import org.junit.runner.RunWith;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.api.mockito.expectation.PowerMockitoStubber;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

@RunWith(PowerMockRunner.class)
public class OnlineAuthenticationDiffblueTest {

  @Rule public final ExpectedException thrown = ExpectedException.none();
  @Rule public final Timeout globalTimeout = new Timeout(10000);

  /* testedClasses: OnlineAuthentication */
  // Test generated by Diffblue Deeptest.
  @PrepareForTest(
      fullyQualifiedNames = {
        "com.kerio.im.core.api.service.AuthService$Credentials",
        "com.kerio.im.core.api.service.AuthService$Connection"
      },
      value = {ExtService.class})
  @Test
  public void authInputNotNullNotNullOutputFalse() throws Exception {

    // Arrange
    final OnlineAuthentication objectUnderTest =
        (OnlineAuthentication) Reflector.getInstance("com.kerio.im.core.auth.OnlineAuthentication");
    final ServiceManager serviceManager =
        (ServiceManager) Reflector.getInstance("com.kerio.im.core.service.ServiceManager");
    final ServiceProcessor serviceProcessor =
        (ServiceProcessor) Reflector.getInstance("com.kerio.im.core.service.ServiceProcessor");
    Reflector.setField(serviceProcessor, "security", null);
    Reflector.setField(serviceProcessor, "status", null);
    final HashMap hashMap = new HashMap();
    Reflector.setField(serviceProcessor, "serviceToPort", hashMap);
    final XmppPorts xmppPorts =
        (XmppPorts) Reflector.getInstance("com.kerio.im.xmpp.api.XmppPorts");
    Reflector.setField(serviceProcessor, "xmppPorts", xmppPorts);
    Reflector.setField(serviceManager, "serviceProcessor", serviceProcessor);
    Reflector.setField(serviceManager, "hostname", null);
    Reflector.setField(objectUnderTest, "ports", serviceManager);
    final Users users = (Users) Reflector.getInstance("com.kerio.im.core.user.Users");
    final UserControlList userControlList =
        (UserControlList) Reflector.getInstance("com.kerio.im.core.user.UserControlList");
    final HashMap<String, UserControl> hashMap1 = new HashMap<String, UserControl>();
    final UserControl userControl =
        (UserControl) Reflector.getInstance("com.kerio.im.core.user.UserControl");
    Reflector.setField(userControl, "userCache", null);
    final Domain domain = (Domain) Reflector.getInstance("com.kerio.im.bean.Domain");
    Reflector.setField(userControl, "domain", domain);
    final XmppUsers xmppUsers =
        (XmppUsers) Reflector.getInstance("com.kerio.im.xmpp.api.XmppUsers");
    Reflector.setField(userControl, "xmppUsers", xmppUsers);
    final GroupProvider groupProvider = new GroupProvider();
    Reflector.setField(userControl, "groups", groupProvider);
    hashMap1.put(null, userControl);
    Reflector.setField(userControlList, "userControlMap", hashMap1);
    Reflector.setField(userControlList, "xmppService", null);
    Reflector.setField(userControlList, "extService", null);
    Reflector.setField(users, "list", userControlList);
    Reflector.setField(objectUnderTest, "users", users);
    final ExtService extService = PowerMockito.mock(ExtService.class);
    final Method authMethod =
        DTUMemberMatcher.method(
            ExtService.class,
            "auth",
            String.class,
            byte[].class,
            String.class,
            String.class,
            boolean.class);
    PowerMockito.doReturn(true)
        .when(extService, authMethod)
        .withArguments(
            or(isA(String.class), isNull(String.class)),
            or(isA(byte[].class), isNull(byte[].class)),
            or(isA(String.class), isNull(String.class)),
            or(isA(String.class), isNull(String.class)),
            anyBoolean());
    Reflector.setField(objectUnderTest, "extService", extService);
    final Credentials credentials = PowerMockito.mock(Credentials.class);
    final Method getUsernameMethod = DTUMemberMatcher.method(Credentials.class, "getUsername");
    PowerMockito.doReturn("^???|<").when(credentials, getUsernameMethod).withNoArguments();
    final byte[] byteArray = {};
    final Method getPassMethod = DTUMemberMatcher.method(Credentials.class, "getPass");
    PowerMockito.doReturn(byteArray).when(credentials, getPassMethod).withNoArguments();
    final Method getLoginMethod = DTUMemberMatcher.method(Credentials.class, "getLogin");
    ((PowerMockitoStubber) PowerMockito.doReturn("    ").doReturn("  "))
        .when(credentials, getLoginMethod)
        .withNoArguments();
    final Method getDomainMethod = DTUMemberMatcher.method(Credentials.class, "getDomain");
    PowerMockito.doReturn("zZZZC").when(credentials, getDomainMethod).withNoArguments();
    final Connection connection = PowerMockito.mock(Connection.class);
    final Method getRemoteIPMethod = DTUMemberMatcher.method(Connection.class, "getRemoteIP");
    PowerMockito.doReturn(" ").when(connection, getRemoteIPMethod).withNoArguments();
    final Method getRemotePortMethod = DTUMemberMatcher.method(Connection.class, "getRemotePort");
    PowerMockito.doReturn(1).when(connection, getRemotePortMethod).withNoArguments();
    final Method getLocalPortMethod = DTUMemberMatcher.method(Connection.class, "getLocalPort");
    PowerMockito.doReturn(0).when(connection, getLocalPortMethod).withNoArguments();
    final Method isLocalPortSecureMethod =
        DTUMemberMatcher.method(Connection.class, "isLocalPortSecure");
    PowerMockito.doReturn(false).when(connection, isLocalPortSecureMethod).withNoArguments();

    // Act
    final boolean retval = objectUnderTest.auth(credentials, connection);

    // result
    assertEquals(false, retval);
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest(
      fullyQualifiedNames = {
        "com.kerio.im.core.api.service.AuthService$Credentials",
        "com.kerio.im.core.api.service.AuthService$Connection"
      },
      value = {ExtService.class})
  @Test
  public void authInputNotNullNotNullOutputFalse2() throws Exception {

    // Arrange
    final OnlineAuthentication objectUnderTest =
        (OnlineAuthentication) Reflector.getInstance("com.kerio.im.core.auth.OnlineAuthentication");
    final ServiceManager serviceManager =
        (ServiceManager) Reflector.getInstance("com.kerio.im.core.service.ServiceManager");
    final ServiceProcessor serviceProcessor =
        (ServiceProcessor) Reflector.getInstance("com.kerio.im.core.service.ServiceProcessor");
    Reflector.setField(serviceProcessor, "security", null);
    Reflector.setField(serviceProcessor, "status", null);
    final HashMap hashMap = new HashMap();
    Reflector.setField(serviceProcessor, "serviceToPort", hashMap);
    final XmppPorts xmppPorts =
        (XmppPorts) Reflector.getInstance("com.kerio.im.xmpp.api.XmppPorts");
    Reflector.setField(serviceProcessor, "xmppPorts", xmppPorts);
    Reflector.setField(serviceManager, "serviceProcessor", serviceProcessor);
    Reflector.setField(serviceManager, "hostname", null);
    Reflector.setField(objectUnderTest, "ports", serviceManager);
    final Users users = (Users) Reflector.getInstance("com.kerio.im.core.user.Users");
    final UserControlList userControlList =
        (UserControlList) Reflector.getInstance("com.kerio.im.core.user.UserControlList");
    final HashMap<String, UserControl> hashMap1 = new HashMap<String, UserControl>();
    final UserControl userControl =
        (UserControl) Reflector.getInstance("com.kerio.im.core.user.UserControl");
    Reflector.setField(userControl, "userCache", null);
    final Domain domain = (Domain) Reflector.getInstance("com.kerio.im.bean.Domain");
    Reflector.setField(userControl, "domain", domain);
    final XmppUsers xmppUsers =
        (XmppUsers) Reflector.getInstance("com.kerio.im.xmpp.api.XmppUsers");
    Reflector.setField(userControl, "xmppUsers", xmppUsers);
    final GroupProvider groupProvider = new GroupProvider();
    Reflector.setField(userControl, "groups", groupProvider);
    hashMap1.put(null, userControl);
    Reflector.setField(userControlList, "userControlMap", hashMap1);
    Reflector.setField(userControlList, "xmppService", null);
    Reflector.setField(userControlList, "extService", null);
    Reflector.setField(users, "list", userControlList);
    Reflector.setField(objectUnderTest, "users", users);
    final ExtService extService = PowerMockito.mock(ExtService.class);
    final Method authMethod =
        DTUMemberMatcher.method(
            ExtService.class,
            "auth",
            String.class,
            byte[].class,
            String.class,
            String.class,
            boolean.class);
    PowerMockito.doReturn(false)
        .when(extService, authMethod)
        .withArguments(
            or(isA(String.class), isNull(String.class)),
            or(isA(byte[].class), isNull(byte[].class)),
            or(isA(String.class), isNull(String.class)),
            or(isA(String.class), isNull(String.class)),
            anyBoolean());
    Reflector.setField(objectUnderTest, "extService", extService);
    final Credentials credentials = PowerMockito.mock(Credentials.class);
    final Method getLoginMethod = DTUMemberMatcher.method(Credentials.class, "getLogin");
    ((PowerMockitoStubber) PowerMockito.doReturn("    ").doReturn(""))
        .when(credentials, getLoginMethod)
        .withNoArguments();
    final byte[] byteArray = {};
    final Method getPassMethod = DTUMemberMatcher.method(Credentials.class, "getPass");
    PowerMockito.doReturn(byteArray).when(credentials, getPassMethod).withNoArguments();
    final Connection connection = PowerMockito.mock(Connection.class);
    final Method getRemoteIPMethod = DTUMemberMatcher.method(Connection.class, "getRemoteIP");
    PowerMockito.doReturn(" ").when(connection, getRemoteIPMethod).withNoArguments();
    final Method getRemotePortMethod = DTUMemberMatcher.method(Connection.class, "getRemotePort");
    PowerMockito.doReturn(1).when(connection, getRemotePortMethod).withNoArguments();
    final Method getLocalPortMethod = DTUMemberMatcher.method(Connection.class, "getLocalPort");
    PowerMockito.doReturn(0).when(connection, getLocalPortMethod).withNoArguments();
    final Method isLocalPortSecureMethod =
        DTUMemberMatcher.method(Connection.class, "isLocalPortSecure");
    PowerMockito.doReturn(false).when(connection, isLocalPortSecureMethod).withNoArguments();

    // Act
    final boolean retval = objectUnderTest.auth(credentials, connection);

    // result
    assertEquals(false, retval);
  }
}
