/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.core.cert;

import com.kerio.im.core.api.service.MessageService;
import com.kerio.im.core.remote.CommandEvent;
import java.util.HashMap;
import java.util.Map;
import org.junit.Test;
import static org.mockito.Mockito.*;
import static org.junit.Assert.*;

/**
 *
 * @author tskoda
 */
public class CertificateEventSyncTest {

    /**
     * Test of event method, of class CertificateEventSync.
     */
    @Test
    public void testEvent() {
        Certificates certificates = mock(Certificates.class);
        CertificateEventSync eventSync = new CertificateEventSync(certificates);

        {
            Map<String, String> content = new HashMap<>();
            content.put(CertificateEventSync.Content.event.namePath, CertificateEventSync.Operation.added.name());
            content.put(CertificateEventSync.Content.id.namePath, "cert1");
            CommandEvent command = new CommandEvent("1", CertificateEventSync.COMMAND, content, new HashMap<String, String>(), mock(MessageService.ResponseHandler.class));

            eventSync.event(command);

            verify(certificates, times(1)).addCertificate(eq("cert1"));
        }

        {
            Map<String, String> content = new HashMap<>();
            content.put(CertificateEventSync.Content.event.namePath, CertificateEventSync.Operation.removed.name());
            content.put(CertificateEventSync.Content.id.namePath, "cert1");
            CommandEvent command = new CommandEvent("1", CertificateEventSync.COMMAND, content, new HashMap<String, String>(), mock(MessageService.ResponseHandler.class));

            eventSync.event(command);

            verify(certificates, times(1)).removeCertificate(eq("cert1"));
        }

        {
            Map<String, String> content = new HashMap<>();
            content.put(CertificateEventSync.Content.event.namePath, CertificateEventSync.Operation.changed.name());
            content.put(CertificateEventSync.Content.id.namePath, "cert1");
            CommandEvent command = new CommandEvent("1", CertificateEventSync.COMMAND, content, new HashMap<String, String>(), mock(MessageService.ResponseHandler.class));

            eventSync.event(command);

            verify(certificates, times(1)).updateCertificate(eq("cert1"));
        }

    }

    @Test
    public void testEventMissingId() {
        Certificates certificates = mock(Certificates.class);
        CertificateEventSync eventSync = new CertificateEventSync(certificates);

        Map<String, String> content = new HashMap<>();
        content.put(CertificateEventSync.Content.event.namePath, CertificateEventSync.Operation.added.name());
        //content.put(CertificateEventSync.Content.id.namePath, "cert1");
        CommandEvent command = new CommandEvent("1", CertificateEventSync.COMMAND, content, new HashMap<String, String>(), mock(MessageService.ResponseHandler.class));

        eventSync.event(command);
        
        verifyZeroInteractions(certificates);
        assertTrue(command.isFail());
    }

    @Test
    public void testEventMissingAction() {
        Certificates certificates = mock(Certificates.class);
        CertificateEventSync eventSync = new CertificateEventSync(certificates);

        Map<String, String> content = new HashMap<>();
        //content.put(CertificateEventSync.Content.event.namePath, CertificateEventSync.Operation.added.name());
        content.put(CertificateEventSync.Content.id.namePath, "cert1");
        CommandEvent command = new CommandEvent("1", CertificateEventSync.COMMAND, content, new HashMap<String, String>(), mock(MessageService.ResponseHandler.class));

        eventSync.event(command);

        verifyZeroInteractions(certificates);
        assertTrue(command.isFail());
    }
}
