/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.service;

import com.kerio.im.bean.Port;
import static com.kerio.im.core.service.FilterTestUtil.*;
import java.net.UnknownHostException;
import static org.junit.Assert.*;
import org.junit.Test;

/**
 *
 * @author tskoda
 */
public class CombinedFilterTest {

    public CombinedFilterTest() {
    }

    @Test
    public void testPassNoFilter() throws UnknownHostException {
        CombinedFilter combined = CombinedFilter.without();
        assertTrue(combined.pass(ip(Long.MIN_VALUE)));
        assertTrue(combined.pass(ip(0)));
        assertTrue(combined.pass(ip(Long.MAX_VALUE)));

        combined = CombinedFilter.with((Port.Filter[]) null);
        assertTrue(combined.pass(ip(Long.MIN_VALUE)));
        assertTrue(combined.pass(ip(0)));
        assertTrue(combined.pass(ip(Long.MAX_VALUE)));

        combined = CombinedFilter.with(new Port.Filter[]{});
        assertTrue(combined.pass(ip(Long.MIN_VALUE)));
        assertTrue(combined.pass(ip(0)));
        assertTrue(combined.pass(ip(Long.MAX_VALUE)));
    }

    @Test
    public void testPassOne() throws UnknownHostException {
        CombinedFilter combined = CombinedFilter.without();
        combined.update(new FilterHost("127.0.0.1"), true);
        assertFalse(combined.pass(ip(127, 0, 0, 2)));
        assertTrue(combined.pass(ip(127, 0, 0, 1)));
    }

    @Test
    public void testPassOneDisabled() throws UnknownHostException {
        CombinedFilter combined = CombinedFilter.without();
        combined.update(new FilterHost("127.0.0.1"), false);
        assertFalse(combined.pass(ip(127, 0, 0, 2)));
        assertFalse(combined.pass(ip(127, 0, 0, 1)));
    }

    @Test
    public void testPassMultiple() throws UnknownHostException {
        CombinedFilter combined = CombinedFilter.without();
        combined.update(new FilterHost("127.0.0.1"), true);
        combined.update(new FilterHost("10.10.10.1"), true);
        combined.update(new FilterHost("127.0.0.2"), true);
        assertTrue(combined.pass(ip(127, 0, 0, 1)));
        assertTrue(combined.pass(ip(127, 0, 0, 2)));
        assertFalse(combined.pass(ip(127, 0, 0, 3)));
        assertTrue(combined.pass(ip(10, 10, 10, 1)));

        combined = CombinedFilter.without();
        combined.update(new FilterHost("127.0.0.1"), false);
        combined.update(new FilterHost("10.10.10.1"), true);
        combined.update(new FilterHost("127.0.0.2"), false);
        assertFalse(combined.pass(ip(127, 0, 0, 1)));
        assertFalse(combined.pass(ip(127, 0, 0, 2)));
        assertFalse(combined.pass(ip(127, 0, 0, 3)));
        assertTrue(combined.pass(ip(10, 10, 10, 1)));

        combined = CombinedFilter.without();
        combined.update(new FilterHost("10.10.10.1"), true);
        combined.update(new FilterHost("127.0.0.1"), false);
        combined.update(new FilterHost("10.10.10.2"), true);
        assertFalse(combined.pass(ip(127, 0, 0, 2)));
        assertFalse(combined.pass(ip(127, 0, 0, 1)));
        assertTrue(combined.pass(ip(10, 10, 10, 1)));
        assertTrue(combined.pass(ip(10, 10, 10, 2)));

        combined = CombinedFilter.without();
        combined.update(new FilterHost("10.10.10.1"), false);
        combined.update(new FilterHost("127.0.0.1"), false);
        combined.update(new FilterHost("10.10.10.2"), false);
        assertFalse(combined.pass(ip(127, 0, 0, 2)));
        assertFalse(combined.pass(ip(127, 0, 0, 1)));
        assertFalse(combined.pass(ip(10, 10, 10, 1)));
    }
    
    @Test
    public void testJustUnknownFilter() throws UnknownHostException {
        CombinedFilter filter = CombinedFilter.without();
        assertTrue(filter.pass(ip(127, 0, 0, 2)));
        assertTrue(filter.pass(ip(10, 10, 10, 1)));
        
        filter.updateUnknown();
        
        assertFalse(filter.pass(ip(127, 0, 0, 2)));
        assertFalse(filter.pass(ip(10, 10, 10, 1)));
    }
    
    @Test
    public void testAlsoUnknownFilter() throws UnknownHostException {
        CombinedFilter filter = CombinedFilter.without();
        
        assertTrue(filter.pass(ip(127, 0, 0, 2)));
        
        filter.updateUnknown();
        
        assertFalse(filter.pass(ip(127, 0, 0, 2)));
        
        filter.update(new FilterHost("127.0.0.2"), true);
        filter.updateUnknown();
        
        assertTrue(filter.pass(ip(127, 0, 0, 2)));

    }
    
    @Test
    public void testIPv4FilterOnly() throws UnknownHostException {
        CombinedFilter filter = CombinedFilter.with(new FilterHost("192.168.1.1"));
        
        assertTrue(filter.pass(ip(192,168,1,1)));
        assertFalse(filter.pass(ip("2001:0718:1803:0044:aa20:66ff:fe59:eb08")));
    }
    
    @Test
    public void testIPv6FilterOnly() throws UnknownHostException {
        CombinedFilter filter = CombinedFilter.with(new FilterHost("2001:0718:1803:0044:aa20:66ff:fe59:eb08"));
        
        assertFalse(filter.pass(ip(192,168,1,1)));
        assertTrue(filter.pass(ip("2001:0718:1803:0044:aa20:66ff:fe59:eb08")));
    }
    
    @Test
    public void testIPv4AndIPv6Combined() throws UnknownHostException {
        CombinedFilter filter = CombinedFilter.with(
                new FilterHost("2001:0718:1803:0044:aa20:66ff:fe59:eb08"),
                new FilterHost("192.168.1.1"));
        
        assertTrue(filter.pass(ip(192,168,1,1)));
        assertTrue(filter.pass(ip("2001:0718:1803:0044:aa20:66ff:fe59:eb08")));
    }
    
    
}
