/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.core.service;

import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author tskoda
 */
public class ConnectionLimitTest {

    public ConnectionLimitTest() {
    }

    @Test
    public void testWithout() {
        ConnectionLimit limit = ConnectionLimit.without();
        //pass everything
        assertTrue(limit.pass());
        limit.connected();
        assertTrue(limit.pass());
        limit.connected();
        limit.connected();
        assertTrue(limit.pass());

        //pass after disconnecting
        limit.disconnected();
        assertTrue(limit.pass());
        //pass even if more disconnects than connects
        limit.disconnected();
        limit.disconnected();
        limit.disconnected();
        limit.disconnected();
        assertTrue(limit.pass());
    }

    @Test
    public void testOne() {
        ConnectionLimit limit = ConnectionLimit.one();
        assertTrue(limit.pass());

        limit.connected(); //connected 1
        assertFalse(limit.pass()); //one is already connected

        limit.connected(); //connected 2
        assertFalse(limit.pass());

        limit.disconnected(); //connected 1
        assertFalse(limit.pass());

        //pass even if more disconnects than connects
        limit.disconnected(); //connected 0
        assertTrue(limit.pass());

        limit.disconnected();
        limit.disconnected(); //connected should be still 0
        assertTrue(limit.pass());

        limit.connected(); //connected 1
        assertFalse(limit.pass());
    }

    @Test
    public void testMaximumNegative() {

        ConnectionLimit limit = ConnectionLimit.maximum(-1);
        assertFalse(limit.pass());

        limit.connected(); //connected 1
        assertFalse(limit.pass());

        limit.disconnected(); //connected 0
        assertFalse(limit.pass());

        limit.disconnected();
        limit.disconnected(); //connected 0
        assertFalse(limit.pass());
    }

    @Test
    public void testMaximum() {

        ConnectionLimit limit = ConnectionLimit.maximum(3);
        assertTrue(limit.pass()); //connected 0

        limit.connected();
        limit.connected(); //connected 2
        assertTrue(limit.pass());

        limit.connected(); //connected 3
        assertFalse(limit.pass());

        limit.disconnected();
        assertTrue(limit.pass());
    }

    @Test
    public void testUpdateInt() {

        ConnectionLimit limit = ConnectionLimit.without();
        assertEquals(0, limit.getLimit());
        limit.connected();
        limit.connected();
        limit.connected();
        assertTrue(limit.pass());

        limit.update(3); //update doesnt affect current count
        assertEquals(3, limit.getLimit());
        assertFalse(limit.pass());

        limit.disconnected();
        assertTrue(limit.pass());

    }

    @Test
    public void testUpdateConnectionLimit() {
        ConnectionLimit limit = ConnectionLimit.without();
        assertEquals(0, limit.getLimit());
        limit.connected();
        limit.connected();
        limit.connected();
        assertTrue(limit.pass());

        limit.update(ConnectionLimit.maximum(3));
        assertEquals(3, limit.getLimit());
        assertFalse(limit.pass());

        limit.disconnected();
        assertTrue(limit.pass());
    }

    @Test
    public void testGetLimit() {

        assertEquals(-1, ConnectionLimit.maximum(-1).getLimit());
        assertEquals(0, ConnectionLimit.maximum(0).getLimit());
        assertEquals(3, ConnectionLimit.maximum(3).getLimit());

        assertEquals(0, ConnectionLimit.without().getLimit());

        assertEquals(1, ConnectionLimit.one().getLimit());

    }
}
