/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.external;

import com.fasterxml.jackson.annotation.JsonProperty;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 *
 * @author vbystricky
 */
public class IMConversation {

    public enum ConversationEvent {
        Created,
        Updated,
        Delivered,
        Read
    }
    @JsonProperty(value = "conversationId")
    Integer conversationId;
    @JsonProperty(value = "receivedUnreadCount")
    Integer receivedUnreadCount;
    @JsonProperty(value = "receivedLastReadId")
    Integer receivedLastReadId;
    @JsonProperty(value = "contacts")
    List<IMContactID> contacts;
    @JsonProperty(value = "event")
    ConversationEvent event;
    @JsonProperty(value = "muted")
    Boolean muted;
    @JsonProperty(value = "lastActivity")
    String lastActivity;
    @JsonProperty(value = "sentLastReadId")
    Integer sentLastReadId;
    @JsonProperty(value = "sentLastDeliveredId")
    Integer sentLastDeliveredId;

    public IMConversation() {

    }

    public IMConversation(
            Integer conversationId,
            Integer receivedUnreadCount,
            Integer receivedLastReadId,
            List<IMContactID> contacts,
            ConversationEvent event,
            Boolean muted,
            String lastActivity,
            Integer sentLastReadId,
            Integer sentLastDeliveredId
    ) {
        this.conversationId = conversationId;
        this.receivedUnreadCount = receivedUnreadCount;
        this.receivedLastReadId = receivedLastReadId;
        setContacts(contacts);
        this.event = event;
        this.muted = muted;
        this.lastActivity = lastActivity;
        this.sentLastReadId = sentLastReadId;
        this.sentLastDeliveredId = sentLastReadId;
    }

    public Integer getConversationId() {
        return conversationId;
    }

    public void setConversationId(Integer conversationId) {
        this.conversationId = conversationId;
    }

    public Integer getReceivedUnreadCount() {
        return receivedUnreadCount;
    }

    public void setReceivedUnreadCount(Integer receivedUnreadCount) {
        this.receivedUnreadCount = receivedUnreadCount;
    }

    public Integer getReceivedLastReadId() {
        return receivedLastReadId;
    }

    public void setReceivedLastReadId(Integer receivedLastReadId) {
        this.receivedLastReadId = receivedLastReadId;
    }

    public List<IMContactID> getContacts() {
        return contacts;
    }

    public final void setContacts(List<IMContactID> contacts) {
        if (contacts == null) {
            this.contacts = new ArrayList<>();
        } else {
            this.contacts = contacts;
        }
    }

    public ConversationEvent getEvent() {
        return event;
    }

    public void setEvent(ConversationEvent event) {
        this.event = event;
    }

    public Boolean isMuted() {
        return muted;
    }

    public void setMuted(Boolean muted) {
        this.muted = muted;
    }

    public String getLastActivity() {
        return lastActivity;
    }

    public void setLastActivity(String lastActivity) {
        this.lastActivity = lastActivity;
    }

    public Integer getSentLastReadId() {
        return sentLastReadId;
    }

    public void setSentLastReadId(Integer sentLastReadId) {
        this.sentLastReadId = sentLastReadId;
    }

    public Integer getSentLastDeliveredId() {
        return sentLastDeliveredId;
    }

    public void setSentLastDeliveredId(Integer sentLastDeliveredId) {
        this.sentLastDeliveredId = sentLastDeliveredId;
    }

    @Override
    public String toString() {
        return "ID: " + conversationId + " Contacts: " + Arrays.toString(contacts.toArray());
    }
}
