package com.kerio.im.connect;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.mockito.AdditionalMatchers.or;
import static org.mockito.Matchers.isA;
import static org.mockito.Matchers.isNull;

import com.diffblue.deeptestutils.Reflector;
import com.diffblue.deeptestutils.mock.DTUMemberMatcher;
import com.kerio.lib.json.api.client.KJsonRpcClient;
import com.kerio.lib.json.api.client.KJsonRpcSession;
import java.lang.reflect.Method;
import org.json.JSONObject;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import org.junit.rules.Timeout;
import org.junit.runner.RunWith;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.api.mockito.expectation.PowerMockitoStubber;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

@RunWith(PowerMockRunner.class)
public class ApiClientDiffblueTest {

  @Rule public final ExpectedException thrown = ExpectedException.none();
  @Rule public final Timeout globalTimeout = new Timeout(10000);

  /* testedClasses: ApiClient */
  // Test generated by Diffblue Deeptest.

  @Test
  public void getClientOutputNull() throws Exception {

    // Arrange
    final ApiClient objectUnderTest =
        (ApiClient) Reflector.getInstance("com.kerio.im.connect.ApiClient");
    Reflector.setField(objectUnderTest, "fileDownload", null);
    Reflector.setField(objectUnderTest, "session", null);
    Reflector.setField(objectUnderTest, "client", null);

    // Act
    final KJsonRpcClient retval = objectUnderTest.getClient();

    // result
    assertNull(retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void getFileDownloadOutputNull() throws Exception {

    // Arrange
    final ApiClient objectUnderTest =
        (ApiClient) Reflector.getInstance("com.kerio.im.connect.ApiClient");
    Reflector.setField(objectUnderTest, "fileDownload", null);
    Reflector.setField(objectUnderTest, "session", null);
    Reflector.setField(objectUnderTest, "client", null);

    // Act
    final FileApiClient retval = objectUnderTest.getFileDownload();

    // result
    assertNull(retval);
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest(fullyQualifiedNames = {"com.kerio.im.connect.ApiClient$KJsonRpcSessionPrivate"})
  @Test
  public void getHostnameOutputNotNull() throws Exception {

    // Arrange
    final ApiClient objectUnderTest =
        (ApiClient) Reflector.getInstance("com.kerio.im.connect.ApiClient");
    Reflector.setField(objectUnderTest, "fileDownload", null);
    final ApiClient.KJsonRpcSessionPrivate kJsonRpcSessionPrivate =
        PowerMockito.mock(ApiClient.KJsonRpcSessionPrivate.class);
    final Method getHostnameMethod =
        DTUMemberMatcher.method(ApiClient.KJsonRpcSessionPrivate.class, "getHostname");
    PowerMockito.doReturn("").when(kJsonRpcSessionPrivate, getHostnameMethod).withNoArguments();
    Reflector.setField(kJsonRpcSessionPrivate, "this$0", null);
    Reflector.setField(objectUnderTest, "session", kJsonRpcSessionPrivate);
    Reflector.setField(objectUnderTest, "client", null);

    // Act
    final String retval = objectUnderTest.getHostname();

    // result
    assertEquals("", retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void getSessionOutputNull() throws Exception {

    // Arrange
    final ApiClient objectUnderTest =
        (ApiClient) Reflector.getInstance("com.kerio.im.connect.ApiClient");
    Reflector.setField(objectUnderTest, "fileDownload", null);
    Reflector.setField(objectUnderTest, "session", null);
    Reflector.setField(objectUnderTest, "client", null);

    // Act
    final KJsonRpcSession retval = objectUnderTest.getSession();

    // result
    assertNull(retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void hostnameInputNotNullPositiveFalseOutputNotNull() throws Exception {

    // Arrange
    final String server = "uuuuuu8u";
    final Integer port = 9;
    final Boolean secure = false;

    // Act
    final Class<?> classUnderTest = Reflector.forName("com.kerio.im.connect.ApiClient");
    final Method methodUnderTest =
        classUnderTest.getDeclaredMethod(
            "hostname",
            Reflector.forName("java.lang.String"),
            Reflector.forName("java.lang.Integer"),
            Reflector.forName("java.lang.Boolean"));
    methodUnderTest.setAccessible(true);
    final String retval = (String) methodUnderTest.invoke(null, server, port, secure);

    // result
    assertEquals("http://uuuuuu8u:9", retval);
  }

  // Test generated by Diffblue Deeptest.

  @Test
  public void hostnameInputNotNullPositiveTrueOutputNotNull() throws Exception {

    // Arrange
    final String server = "r";
    final Integer port = 2;
    final Boolean secure = true;

    // Act
    final Class<?> classUnderTest = Reflector.forName("com.kerio.im.connect.ApiClient");
    final Method methodUnderTest =
        classUnderTest.getDeclaredMethod(
            "hostname",
            Reflector.forName("java.lang.String"),
            Reflector.forName("java.lang.Integer"),
            Reflector.forName("java.lang.Boolean"));
    methodUnderTest.setAccessible(true);
    final String retval = (String) methodUnderTest.invoke(null, server, port, secure);

    // result
    assertEquals("https://r:2", retval);
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest(JSONObject.class)
  @Test
  public void isSessionTimeoutInputNotNullOutputFalse() throws Exception {

    // Arrange
    final ApiClient.KJsonRpcSessionPrivate objectUnderTest =
        (ApiClient.KJsonRpcSessionPrivate)
            Reflector.getInstance("com.kerio.im.connect.ApiClient$KJsonRpcSessionPrivate");
    Reflector.setField(objectUnderTest, "this$0", null);
    final JSONObject message = PowerMockito.mock(JSONObject.class);
    final Method hasMethod = DTUMemberMatcher.method(JSONObject.class, "has", String.class);
    PowerMockito.doReturn(false)
        .when(message, hasMethod)
        .withArguments(or(isA(String.class), isNull(String.class)));

    // Act
    final Class<?> classUnderTest =
        Reflector.forName("com.kerio.im.connect.ApiClient$KJsonRpcSessionPrivate");
    final Method methodUnderTest =
        classUnderTest.getDeclaredMethod(
            "isSessionTimeout", Reflector.forName("org.json.JSONObject"));
    methodUnderTest.setAccessible(true);
    final boolean retval = (boolean) methodUnderTest.invoke(objectUnderTest, message);

    // result
    assertEquals(false, retval);
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest(JSONObject.class)
  @Test
  public void isSessionTimeoutInputNotNullOutputFalse2() throws Exception {

    // Arrange
    final ApiClient.KJsonRpcSessionPrivate objectUnderTest =
        (ApiClient.KJsonRpcSessionPrivate)
            Reflector.getInstance("com.kerio.im.connect.ApiClient$KJsonRpcSessionPrivate");
    Reflector.setField(objectUnderTest, "this$0", null);
    final JSONObject message = PowerMockito.mock(JSONObject.class);
    final Method hasMethod = DTUMemberMatcher.method(JSONObject.class, "has", String.class);
    PowerMockito.doReturn(true)
        .when(message, hasMethod)
        .withArguments(or(isA(String.class), isNull(String.class)));
    final JSONObject jSONObject = PowerMockito.mock(JSONObject.class);
    final Method hasMethod1 = DTUMemberMatcher.method(JSONObject.class, "has", String.class);
    PowerMockito.doReturn(false)
        .when(jSONObject, hasMethod1)
        .withArguments(or(isA(String.class), isNull(String.class)));
    final Method getJSONObjectMethod =
        DTUMemberMatcher.method(JSONObject.class, "getJSONObject", String.class);
    PowerMockito.doReturn(jSONObject)
        .when(message, getJSONObjectMethod)
        .withArguments(or(isA(String.class), isNull(String.class)));

    // Act
    final Class<?> classUnderTest =
        Reflector.forName("com.kerio.im.connect.ApiClient$KJsonRpcSessionPrivate");
    final Method methodUnderTest =
        classUnderTest.getDeclaredMethod(
            "isSessionTimeout", Reflector.forName("org.json.JSONObject"));
    methodUnderTest.setAccessible(true);
    final boolean retval = (boolean) methodUnderTest.invoke(objectUnderTest, message);

    // result
    assertEquals(false, retval);
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest(JSONObject.class)
  @Test
  public void isSessionTimeoutInputNotNullOutputTrue() throws Exception {

    // Arrange
    final ApiClient.KJsonRpcSessionPrivate objectUnderTest =
        (ApiClient.KJsonRpcSessionPrivate)
            Reflector.getInstance("com.kerio.im.connect.ApiClient$KJsonRpcSessionPrivate");
    Reflector.setField(objectUnderTest, "this$0", null);
    final JSONObject message = PowerMockito.mock(JSONObject.class);
    final Method hasMethod = DTUMemberMatcher.method(JSONObject.class, "has", String.class);
    PowerMockito.doReturn(true)
        .when(message, hasMethod)
        .withArguments(or(isA(String.class), isNull(String.class)));
    final JSONObject jSONObject = PowerMockito.mock(JSONObject.class);
    final Method hasMethod1 = DTUMemberMatcher.method(JSONObject.class, "has", String.class);
    PowerMockito.doReturn(true)
        .when(jSONObject, hasMethod1)
        .withArguments(or(isA(String.class), isNull(String.class)));
    final Method getIntMethod = DTUMemberMatcher.method(JSONObject.class, "getInt", String.class);
    PowerMockito.doReturn(-32_001)
        .when(jSONObject, getIntMethod)
        .withArguments(or(isA(String.class), isNull(String.class)));
    final Method getJSONObjectMethod =
        DTUMemberMatcher.method(JSONObject.class, "getJSONObject", String.class);
    PowerMockito.doReturn(jSONObject)
        .when(message, getJSONObjectMethod)
        .withArguments(or(isA(String.class), isNull(String.class)));

    // Act
    final Class<?> classUnderTest =
        Reflector.forName("com.kerio.im.connect.ApiClient$KJsonRpcSessionPrivate");
    final Method methodUnderTest =
        classUnderTest.getDeclaredMethod(
            "isSessionTimeout", Reflector.forName("org.json.JSONObject"));
    methodUnderTest.setAccessible(true);
    final boolean retval = (boolean) methodUnderTest.invoke(objectUnderTest, message);

    // result
    assertEquals(true, retval);
  }

  // Test generated by Diffblue Deeptest.
  @PrepareForTest({JSONObject.class, KJsonRpcSession.class, ApiClient.class})
  @Test
  public void sendAndReceiveInputNotNullNotNullOutputNotNull2() throws Exception {

    // Arrange
    final ApiClient.KJsonRpcSessionPrivate objectUnderTest =
        PowerMockito.mock(ApiClient.KJsonRpcSessionPrivate.class);
    PowerMockito.doCallRealMethod().when(objectUnderTest);
    objectUnderTest.sendAndReceive(
        or(isA(JSONObject.class), isNull(JSONObject.class)),
        or(isA(Method.class), isNull(Method.class)));
    PowerMockito.doCallRealMethod().when(objectUnderTest);
    objectUnderTest.reconnect();
    final Method isSessionTimeoutMethod =
        DTUMemberMatcher.method(
            ApiClient.KJsonRpcSessionPrivate.class, "isSessionTimeout", JSONObject.class);
    PowerMockito.doCallRealMethod()
        .when(objectUnderTest, isSessionTimeoutMethod)
        .withArguments(or(isA(JSONObject.class), isNull(JSONObject.class)));
    final JSONObject jSONObject2 = PowerMockito.mock(JSONObject.class);
    final JSONObject jSONObject = PowerMockito.mock(JSONObject.class);
    final Method hasMethod = DTUMemberMatcher.method(JSONObject.class, "has", String.class);
    PowerMockito.doReturn(true)
        .when(jSONObject, hasMethod)
        .withArguments(or(isA(String.class), isNull(String.class)));
    final JSONObject jSONObject1 = PowerMockito.mock(JSONObject.class);
    final Method hasMethod1 = DTUMemberMatcher.method(JSONObject.class, "has", String.class);
    PowerMockito.doReturn(true)
        .when(jSONObject1, hasMethod1)
        .withArguments(or(isA(String.class), isNull(String.class)));
    final Method getIntMethod = DTUMemberMatcher.method(JSONObject.class, "getInt", String.class);
    PowerMockito.doReturn(-32_001)
        .when(jSONObject1, getIntMethod)
        .withArguments(or(isA(String.class), isNull(String.class)));
    final Method getJSONObjectMethod =
        DTUMemberMatcher.method(JSONObject.class, "getJSONObject", String.class);
    PowerMockito.doReturn(jSONObject1)
        .when(jSONObject, getJSONObjectMethod)
        .withArguments(or(isA(String.class), isNull(String.class)));
    final Method sendAndReceiveMethod =
        DTUMemberMatcher.method(
            KJsonRpcSession.class, "sendAndReceive", JSONObject.class, Method.class);
    ((PowerMockitoStubber) PowerMockito.doReturn(jSONObject).doReturn(jSONObject2))
        .when(objectUnderTest, sendAndReceiveMethod)
        .withArguments(
            or(isA(JSONObject.class), isNull(JSONObject.class)),
            or(isA(Method.class), isNull(Method.class)));
    final ApiClient apiClient = PowerMockito.mock(ApiClient.class);
    final Method getNameMethod = DTUMemberMatcher.method(ApiClient.class, "getName");
    PowerMockito.when(apiClient, getNameMethod).withNoArguments().thenReturn("");
    final FileApiClient fileApiClient =
        (FileApiClient) Reflector.getInstance("com.kerio.im.connect.FileApiClient");
    final ApiClient.KJsonRpcSessionPrivate kJsonRpcSessionPrivate =
        PowerMockito.mock(ApiClient.KJsonRpcSessionPrivate.class);
    Reflector.setField(kJsonRpcSessionPrivate, "this$0", null);
    Reflector.setField(fileApiClient, "session", kJsonRpcSessionPrivate);
    Reflector.setField(apiClient, "fileDownload", fileApiClient);
    final ApiClient.KJsonRpcSessionPrivate kJsonRpcSessionPrivate1 =
        PowerMockito.mock(ApiClient.KJsonRpcSessionPrivate.class);
    final ApiClient apiClient1 = PowerMockito.mock(ApiClient.class);
    final FileApiClient fileApiClient1 =
        (FileApiClient) Reflector.getInstance("com.kerio.im.connect.FileApiClient");
    Reflector.setField(fileApiClient1, "session", null);
    Reflector.setField(apiClient1, "fileDownload", fileApiClient1);
    Reflector.setField(apiClient1, "session", null);
    Reflector.setField(apiClient1, "client", null);
    Reflector.setField(kJsonRpcSessionPrivate1, "this$0", apiClient1);
    Reflector.setField(apiClient, "session", kJsonRpcSessionPrivate1);
    final ApiClient.KJsonRpcClientPrivate kJsonRpcClientPrivate =
        (ApiClient.KJsonRpcClientPrivate)
            Reflector.getInstance("com.kerio.im.connect.ApiClient$KJsonRpcClientPrivate");
    final ApiClient apiClient2 = PowerMockito.mock(ApiClient.class);
    final FileApiClient fileApiClient2 =
        (FileApiClient) Reflector.getInstance("com.kerio.im.connect.FileApiClient");
    Reflector.setField(fileApiClient2, "session", null);
    Reflector.setField(apiClient2, "fileDownload", fileApiClient2);
    Reflector.setField(apiClient2, "session", null);
    Reflector.setField(apiClient2, "client", null);
    Reflector.setField(kJsonRpcClientPrivate, "this$0", apiClient2);
    Reflector.setField(apiClient, "client", kJsonRpcClientPrivate);
    Reflector.setField(objectUnderTest, "this$0", apiClient);
    final JSONObject message = PowerMockito.mock(JSONObject.class);
    final Method method = (Method) Reflector.getInstance("java.lang.reflect.Method");
    Reflector.setField(method, "parameterTypes", null);
    Reflector.setField(method, "clazz", null);
    Reflector.setField(method, "name", null);
    Reflector.setField(method, "override", false);

    // Act
    final JSONObject retval = objectUnderTest.sendAndReceive(message, method);

    // result
    assertNotNull(retval);
  }
}
