/*
 * Copyright 2015 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.rpcserver;

import com.fasterxml.jackson.databind.ObjectMapper;
import java.util.Arrays;
import org.eclipse.jetty.server.Server;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.eclipse.jetty.http.HttpVersion;
import org.eclipse.jetty.server.HttpConfiguration;
import org.eclipse.jetty.server.HttpConnectionFactory;
import org.eclipse.jetty.server.ServerConnector;
import org.eclipse.jetty.server.SslConnectionFactory;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.eclipse.jetty.util.ssl.SslContextFactory;

/**
 *
 * @author tskoda
 */
public class JettyServer {

    private static final Logger LOG = Logger.getLogger(JettyServer.class.getName());

    private final Server jetty;
    private final ObjectMapper mapper;
    private final Map<String, RpcMethod> methods;
    ServerConnector connector;

    public JettyServer(ObjectMapper mapper, RpcMethod... methods) {
        this.jetty = new Server();
        this.mapper = mapper;
        this.methods = new HashMap<>();
        Arrays.stream(methods).forEach((value) -> this.methods.put(value.getMethod(), value));
    }

    public void start(Config config) throws Exception {
        LOG.log(Level.FINE, "Starting HTTP server at {}:{}", new Object[]{config.getHost(), config.getPort()});
        synchronized (this.jetty) {
            //this.shutdown();
            HttpConfiguration httpConfig = new HttpConfiguration();
            httpConfig.setSecurePort(config.getPort());
            httpConfig.setSecureScheme("https");

            SslContextFactory sslContextFactory = new SslContextFactory();

            sslContextFactory.setKeyStorePath(getClass().getResource("keystore.jks").toExternalForm());
            sslContextFactory.setKeyStorePassword("123456");

            SslConnectionFactory sslConnectionFactory = new SslConnectionFactory(sslContextFactory, HttpVersion.HTTP_1_1.toString());

            connector = new ServerConnector(
                    this.jetty,
                    sslConnectionFactory,
                    new HttpConnectionFactory(httpConfig));

            connector.setPort(config.getPort());
            connector.setHost(config.getHost());

            this.jetty.addConnector(connector);
            ServletContextHandler context = new ServletContextHandler(ServletContextHandler.SESSIONS);
            context.addServlet(new ServletHolder(new RpcServlet(mapper, methods)), new Endpoint(0).getPath());

            this.jetty.setHandler(context);
            this.jetty.start();
        }

        LOG.info("Starting HTTP server successful");
    }

    public void shutdown() throws Exception {
        LOG.fine("Stopping HTTP server");
        synchronized (this.jetty) {
            this.jetty.stop();
            if (connector != null) {
                this.jetty.removeConnector(connector);
            }
        }
    }

    public void restart(Config config) throws Exception {
        LOG.fine("Restarting HTTP server");
        synchronized (this.jetty) {
            this.shutdown();
            this.start(config);
        }
    }

}
