/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase;

import java.lang.ref.WeakReference;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import tigase.server.ComponentRegistrator;
import tigase.server.Packet;
import tigase.server.ServerComponent;
import tigase.xmpp.JID;

/**
 *
 * @author tskoda
 */
public class ComponentLookup implements ComponentRegistrator {
    private static final String NAME = "ComponentLookup";
    private final LinkedHashMap<Class, WeakReference<? extends ServerComponent>> registeredComponents = new LinkedHashMap<Class, WeakReference<? extends ServerComponent>>();
    
//    public static ComponentLookup instance() {
//        return new ComponentLookup();
//    }
    
    /**
     * Public constructor present only for internal Tigase mechanisms. Use static method instance() instead.
     */
    public ComponentLookup() {
    }
    
    public <T>T lookup(Class<T> type) {
        WeakReference ref = registeredComponents.get(type);
        T tInstance = (ref != null)? (T)ref.get(): null;
        return tInstance;
    }
    
    public List<WeakReference<? extends ServerComponent>> getAll() {
        return new LinkedList<WeakReference<? extends ServerComponent>>(registeredComponents.values());
    }
    
    @Override
    public boolean addComponent(ServerComponent component) {
        registeredComponents.put(component.getClass(), new WeakReference<ServerComponent>(component));
        return true;
    }

    @Override
    public boolean deleteComponent(ServerComponent component) {
        registeredComponents.remove(component.getClass());
        return true;
    }

    @Override
    public JID getComponentId() {
        return JID.jidInstanceNS(NAME);
    }

    @Override
    public String getName() {
        return NAME;
    }

    @Override
    public void initializationCompleted() {
    }

    @Override
    public void processPacket(Packet packet, Queue<Packet> results) {
        results.add(packet);
    }

    @Override
    public void release() {
    }

    @Override
    public void setName(String name) {

    }
    
    
}
