/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase;

import com.kerio.im.core.api.CoreService;
import com.kerio.im.core.api.GlobalKey;
import com.kerio.im.core.api.service.ConfigurationService;
import com.kerio.im.core.api.service.MessageService;
import java.util.HashMap;
import java.util.Map;
import java.util.Queue;
import java.util.logging.Level;
import java.util.logging.Logger;
import tigase.server.AbstractMessageReceiver;
import tigase.server.Message;
import tigase.server.Packet;
import tigase.xmpp.BareJID;
import tigase.xmpp.JID;
import tigase.xmpp.StanzaType;

/**
 *
 * @author tskoda
 */
public class ConnectComponent extends AbstractMessageReceiver {

    /**
     * Identifier of this component.
     */
    public static final String COMPONENT_NAME = "service-im";
    private static final Logger LOG = Logger.getLogger(ConnectComponent.class.getName());
    //private RosterSynchronizerFactory rosterSynchronizerFactory;
    private final Map<String, Object> properties;
    /**
     *
     */
    private CoreService coreService;
    /**
     *
     */
    private MessageFilter messageFilter;

    //private TigaseUsers iusers;
    //private ComponentLookup componentLookup;
    public ConnectComponent() {
        this.properties = new HashMap<String, Object>();
    }

    @Override
    public Map<String, Object> getDefaults(Map<String, Object> params) {
        LOG.log(Level.FINE, "Initializing Kerio Connect component");
        super.getDefaults(params);
        return params;
    }

    @Override
    public void setProperties(Map<String, Object> props) {
        super.setProperties(props);
        this.properties.putAll(props);
    }

    @Override
    public void initializationCompleted() {
        super.initializationCompleted();
        try {
            //TODO check if not null
            this.coreService = (CoreService) this.properties.get(CoreService.class.getName());
            ConfigurationService configuration = this.coreService.configuration();
            this.messageFilter = new MessageFilter(
                    BareJID.bareJIDInstanceNS(configuration.get(GlobalKey.SERVICE_REMOTE_LOGIN), configuration.get(GlobalKey.SERVICE_REMOTE_DOMAIN)),
                    BareJID.bareJIDInstanceNS(ConnectComponent.COMPONENT_NAME, configuration.get(GlobalKey.SERVICE_REMOTE_DOMAIN)));
            
            LOG.log(Level.INFO, "Initializing Kerio Connect component successful"); //begin is in setDefaults

        } catch (Exception e) {
            LOG.log(Level.SEVERE, "Initialization of Connect component failed", e);
            //TODO get Error handle and publish error
        }
    }

    @Override
    public void setName(String name) {
        super.setName(name);
    }

    @Override
    public int processingInThreads() {
        return 1;
    }

    @Override
    public int processingOutThreads() {
        return 1;
    }

    @Override
    public String getDiscoCategoryType() {
        return "generic";
    }

    @Override
    public String getDiscoDescription() {
        return "Kerio Connect";
    }

    @Override
    public void start() {
        super.start();
    }

    @Override
    public void stop() {
        super.stop();
    }

    @Override
    public void release() {
        try {
            super.release();
        } catch (Exception e) {
            LOG.log(Level.SEVERE, "Error occured while closing connect component threads", e);
        }
    }

    @Override
    public void processPacket(Packet packet) {
        if (this.messageFilter.allow(packet)) {
            this.coreService.message().messageReceived(
                    packet.getElemCData("/message/body"),
                    new ResponseHandler(packet.getStanzaFrom(), packet.getStanzaTo(), new PacketSenderImpl()));
        } else {
            //TODO offer error packet
        }
    }

    public PacketSender getPacketSender() {
        return new PacketSenderImpl();
    }

    private static class ResponseHandler implements MessageService.ResponseHandler {

        private static final String NO_THREAD = null;
        private static final String NO_SUBJECT = null;
        private static final String NO_IDENTIFIER = null;
        private final JID from;
        private final JID to;
        private final PacketSender sender;

        public ResponseHandler(JID from, JID to, PacketSender sender) {
            this.from = from;
            this.to = to;
            this.sender = sender;
        }

        @Override
        public void response(String message) {
            Packet response = Message.getMessage(to, from, StanzaType.chat, message, NO_SUBJECT, NO_THREAD, NO_IDENTIFIER);
            this.sender.processOutPacket(response);
        }
    }

    private class PacketSenderImpl implements PacketSender {

        @Override
        public boolean processOutPacket(Packet packet) {
            addOutPacket(packet);
            return true;
        }

        @Override
        public boolean processOutPacket(Queue<Packet> packets) {
            for (Packet packet : packets) {
                addOutPacket(packet);
            }
            return true;
        }
    }

    //TODO move it to core
    protected static class MessageFilter {

        private static final String MESSAGE_NAME = "message";
        private final BareJID from;
        private final BareJID to;

        public MessageFilter(BareJID from, BareJID to) {
            this.from = from;
            this.to = to;
        }

        public boolean allow(final Packet packet) {
            boolean result = false;

            if (packet != null) {
                result = isMessage(packet) && isFrom(packet) && isTo(packet);
            }

            return result;
        }

        protected boolean isFrom(final Packet packet) {
            return from.equals(packet.getStanzaFrom().getBareJID());
        }

        protected boolean isTo(final Packet packet) {
            return to.equals(packet.getStanzaTo().getBareJID());
        }

        protected boolean isMessage(final Packet packet) {
            //TODO is there better mechanism?
            return MESSAGE_NAME.equals(packet.getElemName());
        }
    }
}
