/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.auth;

import java.util.Arrays;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.sasl.SaslException;
import javax.security.sasl.SaslServer;

/**
 *
 * @author tskoda
 */
public class ConnectSaslPlain implements SaslServer {

    private static final Logger LOG = Logger.getLogger(ConnectSaslPlain.class.getName());
    public static final String ENCRYPTION_KEY = "password-encryption";
    public static final String MECHANISM = "PLAIN";
    private Map<? super String, ?> props;
    private boolean authOk = false;
    private CallbackHandler handler;

    public ConnectSaslPlain(Map<? super String, ?> props, CallbackHandler handler) {
        this.props = props;
        this.handler = handler;
    }

    @Override
    public void dispose() throws SaslException {
        this.props = null;
        this.authOk = false;
        this.handler = null;
    }
    
    

    @Override
    public byte[] evaluateResponse(byte[] data) throws SaslException {
        //fields are separated with /0
        
        int authorizIndex = 0;

        if ((data == null) || (data.length == 0)) {
            return null;
        }

        while ((authorizIndex < data.length) && (data[authorizIndex] != 0)) {
            ++authorizIndex;
        }

        if (authorizIndex >= data.length) {
            throw new SaslException("Uncomplete data");
        }

        String authoriz = new String(data, 0, authorizIndex);
        int userIndex = ++authorizIndex;

        while ((userIndex < data.length) && (data[userIndex] != 0)) {
            ++userIndex;
        }

        if (userIndex >= data.length) {
            throw new SaslException("Uncomplete data");
        }

        String user = new String(data, authorizIndex, userIndex - authorizIndex);

        LOG.log(Level.FINEST, "SASL userId: {0}", user);
        ++userIndex;

        if (userIndex >= data.length) {
            throw new SaslException("Uncomplete data");
        }

        byte[] passwd = Arrays.copyOfRange(data, userIndex, data.length);
        //LOG.log(Level.FINEST, "SASL userId password: {0}", passwd);

        if (passwd != null) {
            String alg = (String) props.get(ENCRYPTION_KEY);
            if (alg != null) {
                throw new SaslException("Password encrypting algorithm " + alg + " is not supported.");
            }
        }          // end of if (passwd != null)

        try {

            ConnectCallback cCallback = new ConnectCallback(passwd, user);
            handler.handle(new Callback[]{cCallback});

            if (cCallback.isAuthenticated()) {
                authOk = true;
            } else {
                throw new SaslException("Not authenticated");
            }

        } catch (Exception e) {
            LOG.log(Level.FINEST, "SASL authentication error: ", e);
            throw new SaslException("Authorization error.", e);
        }        // end of try-catch

        return null;
    }

    @Override
    public String getAuthorizationID() {
        return null;
    }

    @Override
    public String getMechanismName() {
        return ConnectSaslPlain.MECHANISM;
    }

    @Override
    public Object getNegotiatedProperty(String string) {
        if (isComplete()) {
            throw new IllegalStateException("Authentication exchange hasn't finished yet");
        }
        return null;
    }

    @Override
    public boolean isComplete() {
        return authOk;
    }

    @Override
    public byte[] unwrap(byte[] bytes, int i, int i1) throws SaslException {
        return null;
    }

    @Override
    public byte[] wrap(byte[] bytes, int i, int i1) throws SaslException {
        return null;
    }
}
