/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.tigase.bridge.conversation;

import com.kerio.im.external.IMContactID;
import java.util.Objects;
import tigase.xmpp.JID;

/**
 *
 * @author jzacek
 */
public class ConversationJID {

    private final String local;
    private final String domain;
    private String resource = null;
    long lastetResourceUsedTime = 0;

    public ConversationJID(String local, String domain) {
        if (local == null || domain == null) {
            throw new NullPointerException("Local and domain parts must be non-null value!");
        }
        this.local = local;
        this.domain = domain;
    }

    public ConversationJID(JID jid) {
        if (jid.getLocalpart() == null) {
            throw new NullPointerException("Local part must be non-null value!");
        }
        this.local = jid.getLocalpart();
        this.domain = jid.getDomain();
        this.resource = jid.getResource();
    }

    public ConversationJID(IMContactID id) {
        if (id == null || id.getUserName() == null) {
            throw new NullPointerException("Local part must be non-null value!");
        }
        this.local = id.getUserName();
        this.domain = id.getDomain();
    }

    public synchronized JID toJID() {
        return JID.jidInstanceNS(local, domain, resource);
    }

    @Override
    public synchronized String toString() {
        return local + "@" + domain + (resource != null ? "/" + resource : "");
    }

    @Override
    public synchronized int hashCode() {
        int hash = 5;
        hash = 67 * hash + Objects.hashCode(this.local);
        hash = 67 * hash + Objects.hashCode(this.domain);
        hash = 67 * hash + Objects.hashCode(this.resource);
        return hash;
    }

    @Override
    public synchronized boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final ConversationJID other = (ConversationJID) obj;
        if (!this.local.equalsIgnoreCase(other.local)) {
            return false;
        }
        if (!this.domain.equalsIgnoreCase(other.domain)) {
            return false;
        }
        return this.resource.equalsIgnoreCase(other.resource);
    }

    public synchronized boolean equalsWithoutResource(JID other) {
        if (other == null) {
            return false;
        }
        return this.local.equalsIgnoreCase(other.getLocalpart()) && this.domain.equalsIgnoreCase(other.getDomain());
    }

    public synchronized boolean equalsWithoutResource(ConversationJID other) {
        if (other == null) {
            return false;
        }
        return this.local.equals(other.getLocal()) && this.domain.equals(other.getDomain());
    }

    public synchronized int compareTo(ConversationJID other) {
        return this.toString().compareTo(other.toString());
    }

    public synchronized String getResource() {
        return resource;
    }

    public synchronized void setResource(String resource) {
        if (resource != null) {
            lastetResourceUsedTime = System.nanoTime();
        }
        this.resource = resource;
    }

    public synchronized String getLocal() {
        return local;
    }

    public synchronized String getDomain() {
        return domain;
    }

    public long getLastetResourceUsedTime() {
        return lastetResourceUsedTime;
    }
}
