/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.tigase.bridge.conversation;

import com.kerio.im.external.IMContactID;
import com.kerio.im.external.IMConversation;
import com.kerio.im.im.ImApi;
import com.kerio.im.tigase.bridge.IM.IMServerConnection;
import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author jzacek
 */
public class ConversationManager {

    private final Map<Integer, Conversation> conversations = new HashMap<>();

    private ConversationJID createConversationJID(IMContactID contactId) {
        return new ConversationJID(contactId.getUserName(), contactId.getDomain());
    }

    public synchronized Conversation addConversation(IMConversation imConversation) {
        if (imConversation.getConversationId() == null || imConversation.getContacts() == null || imConversation.getContacts().size() != 2) {
            throw new IllegalArgumentException("Invalid IM conversation with ID " + imConversation.getConversationId());
        }

        Conversation conversation;
        conversation = conversations.get(imConversation.getConversationId());
        if (conversation != null) {
            return conversation;
        }

        ConversationJID user1 = createConversationJID(imConversation.getContacts().get(0));
        ConversationJID user2 = createConversationJID(imConversation.getContacts().get(1));
        conversation = new Conversation(user1, user2, imConversation.getConversationId());
        conversations.put(imConversation.getConversationId(), conversation);
        return conversation;
    }

    public synchronized void removeConversation(Integer id) {
        conversations.remove(id);
    }

    public synchronized Conversation getConversation(Integer id) {
        return conversations.get(id);
    }

    public synchronized Conversation getConversation(ConversationJID user1, ConversationJID user2) {
        if (user1.compareTo(user2) > 0) {
            return getRawConversation(user2, user1);
        } else {
            return getRawConversation(user1, user2);
        }
    }

    public synchronized void clear() {
        conversations.clear();
    }

    private Conversation getRawConversation(ConversationJID user1, ConversationJID user2) {
        Object[] res = conversations.entrySet().stream().filter((e) -> {
            return e.getValue().getUser1().equalsWithoutResource(user1)
                    && e.getValue().getUser2().equalsWithoutResource(user2);
        }).toArray();
        return res.length > 0 ? ((Map.Entry<Integer, Conversation>) res[0]).getValue() : null;
    }
}
