/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.tigase.bridge.xmpp;

import com.kerio.im.external.IMEvent;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;

/**
 *
 * @author jzacek
 * @param <T>
 */
public class EventQueue<T> implements IEventsConsumerEx<T> {

    public static final int DEFAULT_TIMEOUT = 5;

    private final BlockingQueue<T> queue = new LinkedBlockingQueue<>();

    private volatile int lock = 0;

    @Override
    public synchronized void enqueue(T value) throws InterruptedException {
        while (lock > 0) {
            wait();
        }
        queue.put(value);
    }

    @Override
    public synchronized void enqueueUnchecked(T value) throws InterruptedException {
        queue.put(value);
    }

    public int size() {
        return queue.size();
    }

    public T dequeue() throws InterruptedException {
        return queue.poll(5, TimeUnit.SECONDS);
    }

    public synchronized void clear() {
        queue.clear();
    }

    @Override
    public synchronized void lock() {
        lock++;
    }

    @Override
    public synchronized void unlock() throws IllegalStateException {
        lock--;
        if (lock < 0) {
            throw new IllegalStateException("Can't unlock more times than lock...");
        }
        notifyAll();
    }
}
