/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.tigase.bridge.xmpp;

import java.util.UUID;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *
 * @author vbystricky
 */
public class XmppIdGenerator {

    private static final AtomicInteger counter = new AtomicInteger(0);
    private static final String KERIO_CONNECT_CONNECTIONS_RESOURCE = "Kerio_Connect_Client";

    public static String generateConnectionID(String userName, String domain) {
        return IMBridge.IM_BRIDGE_NAME + "_" + userName + "@" + domain + "_" + UUID.randomUUID();
    }

    public static String generateResourceID() {
        return KERIO_CONNECT_CONNECTIONS_RESOURCE;
    }

    public static String generatePacketID() {
        return IMBridge.IM_BRIDGE_NAME + "_" + counter.incrementAndGet();
    }

    private static final Pattern packetIdPattern = Pattern.compile(IMBridge.IM_BRIDGE_NAME + "_[\\p{Digit}]+");
    private static final Pattern connectionIdPattern = Pattern.compile(IMBridge.IM_BRIDGE_NAME + "_(.)+(@)(.)+_[\\p{XDigit}]{8}-[\\p{XDigit}]{4}-[\\p{XDigit}]{4}-[\\p{XDigit}]{4}-[\\p{XDigit}]{12}");

    public static boolean isResourceIdInRightFormat(String resourceId) {
        return resourceId.equalsIgnoreCase(KERIO_CONNECT_CONNECTIONS_RESOURCE);
    }

    public static boolean isPacketIdInRightFormat(String packetId) {
        Matcher matcher = packetIdPattern.matcher(packetId);
        return matcher.matches();
    }

    public static boolean isConnectionIdInRightFormat(String connectionId) {
        if (connectionId == null) {
            return false;
        }
        Matcher matcher = connectionIdPattern.matcher(connectionId);
        return matcher.matches();
    }
}
