/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.configuration.task;

import com.kerio.im.common.Task;
import com.kerio.im.common.error.Code;
import com.kerio.im.xmpp.api.XmppException;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class CheckClassPath implements Task<File> {
    
    private static final Logger LOG = Logger.getLogger(CheckClassPath.class.getName());

    @Override
    public void run(File dataDir) {
        
        try {
            LOG.config("Adding plugin containers");
            ClassPathExtender extender = new ClassPathExtender(dataDir);
            
            extender.addToClassPath("service-im-tigase.jar", "tigase-server.jar");
            System.setProperty("java.class.path", extender.getNewClasspath());

            LOG.log(Level.CONFIG, "Class-path changed: {0}", System.getProperty("java.class.path"));
        } catch (Exception e) {
            throw new XmppException("Adding plugin containers failed", e, Code.CLASS_PATH_ERROR);
        }
    }

    private static class ClassPathExtender {

        private String systemClasspath;
        private String newClasspath;
        private File imDir;

        public ClassPathExtender(File imDir) {
            this.systemClasspath = System.getProperty("java.class.path");
            this.imDir = imDir;
        }

        public void addToClassPath(String... requiredLibraries) throws FileNotFoundException {
            File libraryFolder = null;
            StringBuilder newClasspathBuilder = new StringBuilder(this.systemClasspath);

            for (String requiredLibrary : requiredLibraries) {
                if (!isOnClasspath(requiredLibrary)) {
                    if (libraryFolder == null) {
                        libraryFolder = locateLibraryFolder();
                    }

                    File library = locateLibrary(libraryFolder, requiredLibrary);
                    newClasspathBuilder.append(File.pathSeparator).append(library.getPath());
                }
            }


            this.newClasspath = newClasspathBuilder.toString();
        }

        public String getNewClasspath() {
            return newClasspath;
        }

        protected File locateLibrary(File libraryFolder, String library) throws FileNotFoundException {
            File result = new File(libraryFolder, library);

            if (!result.exists() || !result.canRead() || !result.isFile()) {
                throw new FileNotFoundException("File " + library + " not found or not accessible in " + libraryFolder);
            }

            return result;
        }

        protected File locateLibraryFolder() throws FileNotFoundException {
            File result = new File(imDir, "lib");

            if (!result.exists() || !result.canRead() || !result.isDirectory()) {
                throw new FileNotFoundException("Folder " + result + " not found or not accessible");
            }

            return result;

        }

        protected boolean isOnClasspath(String library) {
            return systemClasspath.contains(library);
        }
    }
}
