/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.configuration.task;

import com.kerio.im.common.Task;
import com.kerio.im.common.error.Code;
import com.kerio.im.core.api.GlobalKey;
import com.kerio.im.core.api.service.ConfigurationService;
import com.kerio.im.tigase.DomainServiceImpl;
import com.kerio.im.tigase.derby.DerbyChecker;
import com.kerio.im.tigase.derby.DerbyHandler;
import com.kerio.im.tigase.derby.version.Version;
import com.kerio.im.xmpp.api.XmppException;
import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class CheckDatabase implements Task<Object> {

    private static final Logger LOG = Logger.getLogger(CheckDatabase.class.getName());
    private final String domain;
    private final File databaseRoot;

    public CheckDatabase(ConfigurationService service) {
        this.domain = service.get(GlobalKey.SERVICE_REMOTE_DOMAIN);
        this.databaseRoot = new File(service.get(GlobalKey.DATA_DIR), DomainServiceImpl.DB_ROOT);
    }

    @Override
    public void run(Object nothing) {
        try {
            LOG.log(Level.CONFIG, "Checking database for domain {0}", this.domain);
            DerbyChecker checker = new DerbyChecker(this.databaseRoot, this.domain);
            try {
                checker.check();
            } catch (DerbyChecker.NotExistException e) {
                LOG.log(Level.CONFIG, "Database for domain {0} not found, creating new one", this.domain);
                DerbyHandler handler = new DerbyHandler(this.databaseRoot, this.domain);
                handler.create();
            } catch (DerbyChecker.ObsoleteSchemaException e) {
                LOG.log(Level.CONFIG, "Database schema {0} is obsolete. Current is {1}.", new Object[]{e.getFound(), Version.CURRENT});
                DerbyHandler handler = new DerbyHandler(this.databaseRoot, this.domain);
                handler.upgradeSchema(e.getFound());
                checker.check(); //check it again against the current schema
            }
        } catch (Exception e) {
            throw new XmppException("Checking database for domain {0} failed", e, Code.RESOURCE_ERROR, this.domain);
        }
    }
//    protected String getConnectionString() throws IOException {
//        File configFile = new File(properties.getProperty("property-file"));
//        Properties tigaseProperties = new Properties();
//        FileInputStream is = null;
//        String result = null;
//
//        try {
//            is = new FileInputStream(configFile);
//            tigaseProperties.load(is);
//            result = tigaseProperties.getProperty("--user-db-uri");
//        } finally {
//            if (is != null) {
//                try {
//                    is.close();
//                } catch (Exception e) {
//                    //nothing to do
//                    LOG.log(Level.WARNING, "Can not close input stream", e);
//                }
//            }
//        }
//
//        return result;
//
//    }
}
