/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.configuration.task;

import com.kerio.im.common.Task;
import com.kerio.im.common.error.Code;
import com.kerio.im.xmpp.api.XmppException;
import java.util.logging.Level;
import java.util.logging.Logger;
import tigase.conf.ConfigItem;
import tigase.conf.ConfigRepositoryIfc;
import tigase.db.TigaseDBException;
import tigase.net.ConnectionType;
import tigase.net.SocketType;

/**
 *
 * @author tskoda
 */
public class InitializePorts implements Task<ConfigRepositoryIfc> {

    private static final Logger LOG = Logger.getLogger(InitializePorts.class.getName());
    private static final int[] DISABLE_ALL = {};
   

    @Override
    public void run(ConfigRepositoryIfc configRepo) {
        LOG.log(Level.CONFIG, "Disabling all ports for components s2s, bosh");
        disablePort(configRepo, "s2s");
        disablePort(configRepo, "bosh");
        disablePort(configRepo, "c2s");
    }

    protected void disablePort(ConfigRepositoryIfc configRepo, String component) {
        try {
            configRepo.addItem(item(component, "ports", DISABLE_ALL));
        } catch (TigaseDBException e) {
            throw new XmppException("Can not turn off ports for {0} component", e, Code.INTERNAL_SERVER_ERROR, component);
        }
    }

    protected void enablePort(ConfigRepositoryIfc configRepo, String component, int port, String domain, String ifc) {

        try {
            String keyPrefix = port + "/";
            configRepo.addItem(item(component, "ports", new int[] {port}));
            configRepo.addItem(item(component, keyPrefix + "type", ConnectionType.accept));
            configRepo.addItem(item(component, keyPrefix + "socket", SocketType.plain));
            configRepo.addItem(item(component, keyPrefix + "required", false));
            configRepo.addItem(item(component, keyPrefix + "remote-host", domain));
            configRepo.addItem(item(component, keyPrefix + "ifc", new String[]{ifc}));
        } catch (TigaseDBException e) {
            throw new XmppException("Can not turn on ports for {0} component", e, Code.INTERNAL_SERVER_ERROR, component);
        }
    }

    
    protected ConfigItem item(String component, String type, Object value) {
        ConfigItem configItem = new ConfigItem();
        configItem.set(component, "connections", type, value);
        return configItem;
    }
}
