/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.plugin;

import com.kerio.im.core.api.service.AuthService;
import java.util.Map;
import java.util.Queue;
import java.util.logging.Level;
import java.util.logging.Logger;
import tigase.db.TigaseDBException;
import tigase.server.Packet;
import tigase.xmpp.BareJID;
import tigase.xmpp.NotAuthorizedException;
import tigase.xmpp.XMPPProcessor;
import tigase.xmpp.XMPPResourceConnection;
import tigase.xmpp.XMPPStopListenerIfc;

/**
 *
 * @author tskoda
 */
public class DisconnectPlugin extends XMPPProcessor implements XMPPStopListenerIfc {

    private static final Logger LOG = Logger.getLogger(DisconnectPlugin.class.getName());
    private static final byte[] NO_PASS = null;
    public static final String ID = "disconnect-listener";
    
    
    private AuthService authService;
    
    public DisconnectPlugin() {
    }

//    @Override
//    public String[] supElements() {
//        return new String[]{ALL};
//    }
//
//    @Override
//    public String[] supNamespaces() {
//        return new String[]{ALL};
//    }

    @Override
    public void init(Map<String, Object> settings) throws TigaseDBException {
        super.init(settings);
        this.authService = ((AuthService)settings.get(AuthService.class.getName()));
    }
    
    @Override
    public String id() {
        return ID;
    }

    @Override
    public void stopped(XMPPResourceConnection session, Queue<Packet> results, Map<String, Object> settings) {
        try {
            if (session.isAuthorized() && !session.isServerSession()) {
                
                BareJID userBareJID = session.getBareJID();
                
                AuthService.Credentials credentials = new AuthService.Credentials(
                        userBareJID.getLocalpart(), 
                        userBareJID.getDomain(), 
                        NO_PASS);
                
                authService.disconnected(credentials);
            }
        } catch (NotAuthorizedException e) {
            LOG.log(Level.WARNING, "Disconnecting not authorized user session.", e);
        }
    }
}
