/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.plugin;

import com.kerio.im.core.api.service.AuthService;
import com.kerio.im.tigase.ClientResolver;
import java.util.Map;
import java.util.Queue;
import java.util.logging.Level;
import java.util.logging.Logger;
import tigase.db.NonAuthUserRepository;
import tigase.db.TigaseDBException;
import tigase.server.Packet;
import tigase.xml.Element;
import tigase.xmpp.NoConnectionIdException;
import tigase.xmpp.XMPPResourceConnection;
import tigase.xmpp.impl.SaslAuth;

/**
 *
 * @author tskoda
 */
public class SASLPlugin extends SaslAuth {

    private static final Logger LOG = Logger.getLogger(SASLPlugin.class.getName());
    public static final String PLUGIN_ID = "connect-sasl";
    private ClientResolver clientSession;

    @Override
    public void init(Map<String, Object> settings) throws TigaseDBException {
        super.init(settings);
        this.clientSession = (ClientResolver) settings.get(ClientResolver.class.getName());
    }

    @Override
    public String id() {
        return PLUGIN_ID;
    }
    
    private boolean allow(AuthService.Connection clientConnectionInfo, XMPPResourceConnection session) {
        return (clientConnectionInfo.isLocalPortSecure() || !clientConnectionInfo.isTlsRequired() || (session.getSessionData(TLSPlugin.ID) != null));
    }

    @Override
    public void process(Packet packet, XMPPResourceConnection session, NonAuthUserRepository repo, Queue<Packet> results, Map<String, Object> settings) {
        if (session != null) { //otherwise do nothing
		try {
                AuthService.Connection clientConnectionInfo = this.clientSession.resolve(session.getConnectionId().getResource());
                if (clientConnectionInfo != null) {
                    //process SASL only if tls is not required or proceeded or its ssl connection
                    if  (allow(clientConnectionInfo, session)) {
                        super.process(packet, session, repo, results, settings);
                    }
                } else {
                    LOG.log(Level.INFO, "Failed to proceed SASL authentication, unable to identify client connection");
                    errorResponse(packet, session, results);
                }

            } catch (NoConnectionIdException e) {
                //do nothing, sasl shouldn't be offered anyway
                LOG.log(Level.INFO, "Failed to proceed SASL authentication, unable to identify client session");
                errorResponse(packet, session, results);
            }
        }
    }

    @Override
    public Element[] supStreamFeatures(XMPPResourceConnection session) {
        Element[] result = null;

        if (session != null) {
            try {
                AuthService.Connection clientConnectionInfo = this.clientSession.resolve(session.getConnectionId().getResource());
                //offer sasl only if tls is not required or proceeded or its ssl connection
                if (clientConnectionInfo != null) {
                    if (allow(clientConnectionInfo, session)) {
                        result = super.supStreamFeatures(session);
                    }
                } else {
                    LOG.log(Level.INFO, "Unable to identify user connection, not offering SASL");
                }
            } catch (NoConnectionIdException e) {
                //unknown connection, we offer optional tls
                LOG.log(Level.INFO, "Unable to identify user session, not offering SASL");
                //result = super.supStreamFeatures(session);
            }
        }

        return result;
    }
}
