/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.plugin;

import com.kerio.im.core.api.service.AuthService.Connection;
import com.kerio.im.tigase.ClientResolver;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import tigase.db.TigaseDBException;
import tigase.xml.Element;
import tigase.xmpp.NoConnectionIdException;
import tigase.xmpp.XMPPResourceConnection;
import tigase.xmpp.impl.StartTLS;

/**
 *
 * @author tskoda
 */
public class TLSPlugin extends StartTLS {

    private static final Logger LOG = Logger.getLogger(TLSPlugin.class.getName());
    public static final String PLUGIN_ID = "connect-starttls";
    private ClientResolver clientSession;

    @Override
    public void init(Map<String, Object> settings) throws TigaseDBException {
        super.init(settings);
        this.clientSession = (ClientResolver) settings.get(ClientResolver.class.getName());
    }

    @Override
    public String id() {
        return PLUGIN_ID;
    }

    @Override
    public Element[] supStreamFeatures(XMPPResourceConnection session) {
        Element[] result = null;

        if ((session != null) && (session.getSessionData(ID) == null)) {
            try {
                Connection clientConnectionInfo = this.clientSession.resolve(session.getConnectionId().getResource());
                if (clientConnectionInfo != null) {
                    if (!clientConnectionInfo.isLocalPortSecure()) { //we do not offer TLS over SSL connections
                        if (clientConnectionInfo.isTlsRequired()) {
                            result = F_REQUIRED;
                        } else {
                            result = F_NOT_REQUIRED;
                        }
                    } //else do not offer at all
                } else {
                    LOG.log(Level.INFO, "Unable to identify user connection, offering optional TLS");
                    result = F_NOT_REQUIRED;
                }
            } catch (NoConnectionIdException e) {
                //unknown connection, we offer optional tls
                LOG.log(Level.INFO, "Unable to identify user session, offering optional TLS");
                result = F_NOT_REQUIRED;
            }
        }

        return result;
    }
}
