/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.plugin;

import com.kerio.im.bean.VCard;
import tigase.xml.Element;
import tigase.xml.XMLUtils;

/**
 *
 * @author tskoda
 */
public class VCardWrapper implements VCard {

    public static VCardWrapper instance() {
        return new VCardWrapper(new Element("vCard", new String[]{"xmlns"}, new String[]{"vcard-temp"}));
    }

    private class VCardChild implements VCard {

        VCardWrapper root;
        Element parent;

        public VCardChild(VCardWrapper root, Element parent) {
            this.root = root;
            this.parent = parent;
        }

        @Override
        public VCard put(String element, String value) {
            return raw(element, XMLUtils.escape(value));
        }

        @Override
        public VCard raw(String element, String value) {
            this.root.setNotEmpty();
            Element childElement = new Element(element, value);
            this.parent.addChild(childElement);
            return new VCardChild(this.root, childElement);
        }

        @Override
        public VCard put(String element) {
            this.root.setNotEmpty();
            Element childElement = new Element(element);
            this.parent.addChild(childElement);
            return new VCardChild(this.root, childElement);
        }

        @Override
        public VCard photo(String element, String value, String sha1) {
            this.root.setSha1(sha1);
            return raw(element, value);
        }

        @Override
        public String photoSHA1() {
            throw new UnsupportedOperationException("Not supported");
        }
    }
    
    private final VCardChild root;
    private boolean empty;
    private String sha1;

    private VCardWrapper(Element root) {
        this.root = new VCardChild(this, root);;
        this.empty = true;
    }

    @Override
    public VCard raw(String element, String value) {
        return this.root.raw(element, value);
    }

    @Override
    public VCard put(String element, String value) {
        return this.root.put(element, value);
    }

    @Override
    public VCard put(String element) {
        return this.root.put(element);
    }

    @Override
    public VCard photo(String element, String value, String sha1) {
        return this.root.photo(element, value, sha1);
    }

    @Override
    public String photoSHA1() {
        return sha1;
    }
    
    private void setSha1(String sha1) {
        this.sha1 = sha1;
    }

    private void setNotEmpty() {
        this.empty = false;
    }

    public boolean isEmpty() {
        return this.empty;
    }
    
    public Element root() {
        return this.root.parent;
    }
}
