/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.roster;

import com.kerio.im.bean.User;
import com.kerio.im.core.api.service.GroupService;
import com.kerio.im.tigase.Conversion;
import com.kerio.im.tigase.user.UserMapper;
import com.kerio.im.tigase.user.XmppUsersStore;
import java.util.Arrays;
import tigase.xml.Element;
import tigase.xmpp.BareJID;
import tigase.xmpp.impl.roster.RosterAbstract;

/**
 *
 * @author tskoda
 */
public class ElementMapper implements XmppUsersStore.Mapper<BareJID, RosterContent.Buddy> {

    private final GroupService groups;
    private final UserMapper userMapper;

    public ElementMapper(GroupService groups, UserMapper userMapper) {
        this.groups = groups;
        this.userMapper = userMapper;
    }

    @Override
    public BareJID key(BareJID userBareJID) {
        return userBareJID;
    }

    @Override
    public RosterContent.Buddy value(BareJID userBareJID, String fullName, boolean status, boolean visible) {
        Element element = new PacketBuilder.AddUserPacket(userBareJID, RosterAbstract.SubscriptionType.both, fullName, new String[]{groups.group(userBareJID.toString())}).toElement();
        User user = this.userMapper.value(userBareJID, fullName, status, visible);
        return new RosterContent.Buddy(element, user);
    }

    public RosterContent.Buddy value(User user) {
        Element element = new PacketBuilder.AddUserPacket(Conversion.toBareJID(user), RosterAbstract.SubscriptionType.both, user.getFullName(), new String[]{groups.group(user.getPrimaryEmail())}).toElement();
        return new RosterContent.Buddy(element, user);
    }

    public Element value(User user, PersonalRoster.Buddy buddy) {
        PacketBuilder.AddUserPacket addUserPacket = new PacketBuilder.AddUserPacket(Conversion.toBareJID(user), RosterAbstract.SubscriptionType.both, user.getFullName(), new String[]{groups.group(user.getPrimaryEmail())});
        return addUserPacket.merge(buddy).toElement();
    }
}
