/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.user;

import com.kerio.im.bean.User;
import com.kerio.im.tigase.Conversion;
import com.kerio.im.tigase.roster.IDefaultRosterListener;
import java.util.EnumSet;
import java.util.Map;

/**
 *
 * @author tskoda
 */
public class XmppUsers implements com.kerio.im.xmpp.api.XmppUsers {

    private final XmppUsersStore xmppUsersDB;
    private final IDefaultRosterListener xmppUsersRosters;

    public XmppUsers(XmppUsersStore xmppUsersDB, IDefaultRosterListener xmppUsersRosters) {
        this.xmppUsersDB = xmppUsersDB;
        this.xmppUsersRosters = xmppUsersRosters;
    }

    @Override
    public boolean check(User user) {
        boolean result = this.xmppUsersDB.exist(user.getLogin());
        if (result) {
            XmppUsersStore.Roster roster = this.xmppUsersDB.readRoster(user);

            int sizeBefore = roster.getRoster().size();
            this.xmppUsersRosters.check(user, roster.getRoster());
            int sizeAfter = roster.getRoster().size();

            if (sizeBefore != sizeAfter) {
                this.xmppUsersDB.saveRoster(roster);
            }
        }
        return result;
    }

    @Override
    public Map<String, User> all() {
        return this.xmppUsersDB.all();
    }

    @Override
    public boolean add(User user) {
        this.xmppUsersRosters.add(user);
        return this.xmppUsersDB.add(user);
    }

    @Override
    public boolean remove(User user) {
        this.xmppUsersRosters.remove(user);
        return this.xmppUsersDB.remove(user);
    }

    @Override
    public boolean update(User user, EnumSet<Changes> changes) {
        this.xmppUsersRosters.update(user, changes);
        return this.xmppUsersDB.update(user, changes);
    }

    @Override
    public void repair(User user) {
        //first disconnect user (he is not able to login until repair ends)
        this.xmppUsersDB.disconnect(Conversion.toBareJID(user));

        XmppUsersStore.Roster roster = this.xmppUsersDB.readRoster(user);

        int sizeBefore = roster.getRoster().size();
        this.xmppUsersRosters.repair(user, roster.getRoster());
        int sizeAfter = roster.getRoster().size();

        if (sizeBefore != sizeAfter) {
            this.xmppUsersDB.saveRoster(roster);
        }
    }

    @Override
    public void bye() {
        this.xmppUsersDB.bye();
        this.xmppUsersRosters.bye();
    }
}
