/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.tigase;

import static com.kerio.im.tigase.ResourceSelectorTestUtil.mockSession;
import static com.kerio.im.tigase.ResourceSelectorTestUtil.serviceUnavailible;
import static com.kerio.im.tigase.ResourceSelectorTestUtil.with;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;
import tigase.util.TigaseStringprepException;
import tigase.xmpp.JID;
import tigase.xmpp.ResourceSelector;
import tigase.xmpp.StanzaType;
import tigase.xmpp.XMPPResourceConnection;
import tigase.xmpp.XMPPSession;
import static com.kerio.im.tigase.ResourceSelectorTestUtil.*;
import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;

/**
 *
 * @author tskoda
 */
public class MessageResourceSelectorWithResourceNoMatchTest {

    @Rule
    public ExpectedException exception = ExpectedException.none();
    private MessageResourceSelector selector = new MessageResourceSelector();
    private JID jid = JID.jidInstanceNS("user@test/resource");

    @Test
    public void testEmpty_allNegative() throws Exception {
        StanzaType type = null; // null == normal message
        XMPPSession mockSession = mockSession(
                jid,
                -1,
                with(-100, MessageResourceSelector.Show.availible),
                with(-10, MessageResourceSelector.Show.chat),
                with(-1, MessageResourceSelector.Show.dnd));

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testNormal_allNegative() throws Exception {
        StanzaType type = StanzaType.normal; // null == normal message
        XMPPSession mockSession = mockSession(
                jid,
                -1,
                with(-100, MessageResourceSelector.Show.availible),
                with(-10, MessageResourceSelector.Show.chat),
                with(-1, MessageResourceSelector.Show.dnd));

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testChat_allNegative() throws Exception {
        StanzaType type = StanzaType.chat;
        XMPPSession mockSession = mockSession(
                jid,
                -1,
                with(-100, MessageResourceSelector.Show.availible),
                with(-10, MessageResourceSelector.Show.chat),
                with(-1, MessageResourceSelector.Show.dnd));

        assertThat(selector.preferredResources(jid, type, mockSession).size(), is(0)); //offline delivery
    }

    @Test
    public void testGroupchat_allNegative() throws Exception {
        StanzaType type = StanzaType.groupchat; // null == normal message
        XMPPSession mockSession = mockSession(
                jid,
                -1,
                with(-100, MessageResourceSelector.Show.availible),
                with(-10, MessageResourceSelector.Show.chat),
                with(-1, MessageResourceSelector.Show.dnd));

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testHeadline_allNegative() throws Exception {
        StanzaType type = StanzaType.headline; // null == normal message
        XMPPSession mockSession = mockSession(
                jid,
                -1,
                with(-100, MessageResourceSelector.Show.availible),
                with(-10, MessageResourceSelector.Show.chat),
                with(-1, MessageResourceSelector.Show.dnd));

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testEmpty_empty() throws Exception {
        StanzaType type = null; // null == normal message
        XMPPSession mockSession = mockSession(
                jid,
                -1);

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testNormal_empty() throws Exception {
        StanzaType type = StanzaType.normal; // null == normal message
        XMPPSession mockSession = mockSession(
                jid,
                -1);

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testChat_empty() throws Exception {
        StanzaType type = StanzaType.chat; // null == normal message
        XMPPSession mockSession = mockSession(
                jid,
                -1);

        assertThat(selector.preferredResources(jid, type, mockSession).size(), is(0)); //offline delivery
    }

    @Test
    public void testGroupchat_empty() throws Exception {
        StanzaType type = StanzaType.groupchat; // null == normal message
        XMPPSession mockSession = mockSession(
                jid,
                -1);

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testHeadline_empty() throws Exception {
        StanzaType type = StanzaType.headline; // null == normal message
        XMPPSession mockSession = mockSession(
                jid,
                -1);

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testEmpty_allPositive() throws Exception {
        StanzaType type = null; // null == normal message
        XMPPSession mockSession = mockSession(
                jid,
                -1,
                with(100, MessageResourceSelector.Show.availible),
                with(10, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testNormal_allPositive() throws Exception {
        StanzaType type = StanzaType.normal; // null == normal message
        XMPPSession mockSession = mockSession(
                jid,
                -1,
                with(100, MessageResourceSelector.Show.availible),
                with(10, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testChat_allPositive() throws Exception {
        StanzaType type = StanzaType.chat; // null == normal message
        XMPPSession mockSession = mockSession(
                jid,
                -1,
                with(100, MessageResourceSelector.Show.availible),
                with(10, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        assertThat(selector.preferredResources(jid, type, mockSession).size(), is(1)); //delivery even if its to unexistence resource
    }

    @Test
    public void testGroupchat_allPositive() throws Exception {
        StanzaType type = StanzaType.groupchat; // null == normal message
        XMPPSession mockSession = mockSession(
                jid,
                -1,
                with(100, MessageResourceSelector.Show.availible),
                with(10, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

    @Test
    public void testHeadline_allPositive() throws Exception {
        StanzaType type = StanzaType.headline; // null == normal message
        XMPPSession mockSession = mockSession(
                jid,
                -1,
                with(100, MessageResourceSelector.Show.availible),
                with(10, MessageResourceSelector.Show.chat),
                with(1, MessageResourceSelector.Show.dnd));

        exception.expect(serviceUnavailible());
        selector.preferredResources(jid, type, mockSession);
    }

}
