/*
 * Copyright 2014 Kerio Technologies s.r.o. All rights reserved
 */
package com.kerio.im.tigase;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;
import org.hamcrest.BaseMatcher;
import static org.hamcrest.CoreMatchers.is;
import org.hamcrest.Description;
import org.hamcrest.Matcher;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.eq;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import tigase.util.TigaseStringprepException;
import tigase.xml.Element;
import tigase.xml.XMLNodeIfc;
import tigase.xmpp.JID;
import tigase.xmpp.ResourceSelector;
import tigase.xmpp.XMPPResourceConnection;
import tigase.xmpp.XMPPSession;

/**
 *
 * @author tskoda
 */
public class ResourceSelectorTestUtil {

    public static Matcher<Set<XMPPResourceConnection>> containsOnly(final XMPPResourceConnection connection) {
        return new BaseMatcher<Set<XMPPResourceConnection>>() {

            @Override
            public boolean matches(Object item) {
                Set<XMPPResourceConnection> set = (Set<XMPPResourceConnection>) item;
                return set.size() == 1 && set.contains(connection);
            }

            @Override
            public void describeTo(Description description) {
            }
        };
    }

    public static Matcher<? extends ResourceSelector.CanNotDeliver> serviceUnavailible() {
        return new BaseMatcher<ResourceSelector.CanNotDeliver>() {

            @Override
            public boolean matches(Object item) {
                ResourceSelector.CanNotDeliver exception = (ResourceSelector.CanNotDeliver) item;
                return exception.getAction() == ResourceSelector.CanNotDeliver.Action.serviceUnavailible;
            }

            @Override
            public void describeTo(Description description) {
                description.appendValue(ResourceSelector.CanNotDeliver.Action.serviceUnavailible);
            }
        };
    }

    public static Matcher<? extends ResourceSelector.CanNotDeliver> ignore() {
        return new BaseMatcher<ResourceSelector.CanNotDeliver>() {

            @Override
            public boolean matches(Object item) {
                ResourceSelector.CanNotDeliver exception = (ResourceSelector.CanNotDeliver) item;
                return exception.getAction() == ResourceSelector.CanNotDeliver.Action.ignore;
            }

            @Override
            public void describeTo(Description description) {
                description.appendValue(ResourceSelector.CanNotDeliver.Action.ignore);
            }
        };
    }

    public static XMPPResourceConnection with(int priority, MessageResourceSelector.Show show) {
        XMPPResourceConnection result = mock(XMPPResourceConnection.class);
        when(result.getPriority()).thenReturn(priority);

        Element presence = new Element("presence");
        when(result.getPresence()).thenReturn(presence);
        when(result.isAuthorized()).thenReturn(Boolean.TRUE);

        if (show != MessageResourceSelector.Show.availible) { //availible has no show
            List<XMLNodeIfc> children = new LinkedList<XMLNodeIfc>();
            children.add(new Element("show", show.name()));
            presence.setChildren(children);
        }

        return result;
    }

    public static XMPPSession mockSession(JID jid, int indexOfActive, XMPPResourceConnection... connections) {

        List<XMPPResourceConnection> activeResources = new ArrayList<XMPPResourceConnection>(Arrays.asList(connections));
        XMPPSession mockSession = mock(XMPPSession.class);
        when(mockSession.getActiveResources()).thenReturn(activeResources);

        XMPPResourceConnection active = (indexOfActive >= 0 && connections != null && indexOfActive < connections.length) ? connections[indexOfActive] : null;
        when(mockSession.getResourceForJID(eq(jid))).thenReturn(active);

        return mockSession;

    }

}
