/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.auth;

import java.io.IOException;
import java.util.Arrays;
import java.util.Map;
import java.util.HashMap;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.sasl.SaslException;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author tskoda
 */
public class ConnectSaslPlainTest {

    public ConnectSaslPlainTest() {
    }

    /* - nothing to test
    @Test
    public void testDispose() throws Exception {
    
    }
     */
    @Test
    public void testEvaluateResponseEmptyData() throws Exception {
        Map<? super String, ?> props = new HashMap<String, Object>();
        CallbackHandler handler = new TestCallbackHandler(null, null);

        ConnectSaslPlain saslPlainServer = new ConnectSaslPlain(props, handler);
        byte[] evaluateResponse = saslPlainServer.evaluateResponse(new byte[0]);

        assertNull(evaluateResponse);
        assertFalse(saslPlainServer.isComplete());

    }

    @Test(expected = SaslException.class)
    public void testEvaluateResponseNoName() throws Exception {

        Map<? super String, ?> props = new HashMap<String, Object>();
        CallbackHandler handler = new TestCallbackHandler(null, null);

        ConnectSaslPlain saslPlainServer = new ConnectSaslPlain(props, handler);
        byte[] evaluateResponse = saslPlainServer.evaluateResponse("authoriz".getBytes());

        assertNull(evaluateResponse);
        assertFalse(saslPlainServer.isComplete());

    }

    @Test(expected = SaslException.class)
    public void testEvaluateResponseNoPass() throws Exception {

        Map<? super String, ?> props = new HashMap<String, Object>();
        CallbackHandler handler = new TestCallbackHandler(null, null);

        ConnectSaslPlain saslPlainServer = new ConnectSaslPlain(props, handler);
        byte[] evaluateResponse = saslPlainServer.evaluateResponse("authoriz\0Name".getBytes());

        assertNull(evaluateResponse);
        assertFalse(saslPlainServer.isComplete());

    }

    @Test
    public void testEvaluateResponseAuthenticated() throws Exception {
        Map<? super String, ?> props = new HashMap<String, Object>();
        CallbackHandler handler = new TestCallbackHandler("Name", "Password".getBytes());

        ConnectSaslPlain saslPlainServer = new ConnectSaslPlain(props, handler);
        byte[] evaluateResponse = saslPlainServer.evaluateResponse("authoriz\0Name\0Password".getBytes());

        assertNull(evaluateResponse);
        assertTrue(saslPlainServer.isComplete());
    }

    @Test
    public void testEvaluateResponseWrongName() throws Exception {
        Map<? super String, ?> props = new HashMap<String, Object>();
        CallbackHandler handler = new TestCallbackHandler("Name", "Password".getBytes());

        ConnectSaslPlain saslPlainServer = new ConnectSaslPlain(props, handler);
        try {
            byte[] evaluateResponse = saslPlainServer.evaluateResponse("authoriz\0WrongName\0Password".getBytes());
            fail();
        } catch (SaslException e) {
            //expected
        }

        
        assertFalse(saslPlainServer.isComplete());
    }

    @Test
    public void testEvaluateResponseWrongPass() throws Exception {
        Map<? super String, ?> props = new HashMap<String, Object>();
        CallbackHandler handler = new TestCallbackHandler("Name", "Password".getBytes());

        ConnectSaslPlain saslPlainServer = new ConnectSaslPlain(props, handler);
        try {
            byte[] evaluateResponse = saslPlainServer.evaluateResponse("authoriz\0Name\0AnotherPassword".getBytes());
            fail();
        } catch (SaslException e) {
            //expected
        }

        
        assertFalse(saslPlainServer.isComplete());
    }

    private static class TestCallbackHandler implements CallbackHandler {

        private final byte[] pass;
        private final String user;
        private boolean handleResult;

        private TestCallbackHandler(String user, byte[] pass) {

            this.user = user;
            this.pass = pass;
        }

        @Override
        public void handle(Callback[] clbcks) throws IOException, UnsupportedCallbackException {
            if ((clbcks.length != 1) || !(clbcks[0] instanceof ConnectCallback)) {
                throw new UnsupportedCallbackException(null);
            }

            ConnectCallback cc = (ConnectCallback) clbcks[0];

            this.handleResult = equals(cc.getUserName(), cc.getPassword());
            cc.setAuthenticated(this.handleResult);

        }

        public boolean equals(String user, byte[] pass) {

            if ((this.pass == null) ? (pass != null) : !Arrays.equals(this.pass,pass)) {
                return false;
            }
            if ((this.user == null) ? (user != null) : !this.user.equals(user)) {
                return false;
            }
            return true;
        }
    }
}
