/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.roster;

import java.util.Arrays;
import org.hamcrest.BaseMatcher;
import org.hamcrest.Description;
import org.hamcrest.Matcher;

/**
 *
 * @author tskoda
 */
public class BuddyUtil {


    public Matcher<PersonalRoster.Buddy> name(final String name) {
        return new BaseMatcher<PersonalRoster.Buddy>() {

            @Override
            public boolean matches(Object item) {
                PersonalRoster.Buddy buddy = (PersonalRoster.Buddy) item;
                if (name == null) {
                    return buddy.getName() == null;
                } else {
                    return name.equals(buddy.getName());
                }
            }

            @Override
            public void describeTo(Description description) {
                description.appendValue(name);
            }
        };
    }

    public Matcher<PersonalRoster.Buddy> groups(final String... groups) {
        return new BaseMatcher<PersonalRoster.Buddy>() {

            @Override
            public boolean matches(Object item) {
                PersonalRoster.Buddy buddy = (PersonalRoster.Buddy) item;
                if (groups == null) {
                    return buddy.getGroups() == null;
                } else {
                    return Arrays.equals(groups, buddy.getGroups());
                }
            }

            @Override
            public void describeTo(Description description) {
                description.appendValue(Arrays.toString(groups));
            }
        };
    }

    public Matcher<PersonalRoster.Buddy> added() {
        return new BaseMatcher<PersonalRoster.Buddy>() {

            @Override
            public boolean matches(Object item) {
                PersonalRoster.Buddy buddy = (PersonalRoster.Buddy) item;
                return buddy.isAdded() && ! buddy.isDeleted();
            }

            @Override
            public void describeTo(Description description) {
                description.appendText("added = true, deleted = false");
            }
        };
    }

    public Matcher<PersonalRoster.Buddy> removed() {
        return new BaseMatcher<PersonalRoster.Buddy>() {

            @Override
            public boolean matches(Object item) {
                PersonalRoster.Buddy buddy = (PersonalRoster.Buddy) item;
                return buddy.isDeleted() && ! buddy.isAdded();
            }

            @Override
            public void describeTo(Description description) {
                description.appendText("added = false, deleted = true");
            }
        };
    }
    
    public Matcher<PersonalRoster.Buddy> notManaged() {
        return new BaseMatcher<PersonalRoster.Buddy>() {

            @Override
            public boolean matches(Object item) {
                PersonalRoster.Buddy buddy = (PersonalRoster.Buddy) item;
                return ! buddy.isDeleted() && ! buddy.isAdded();
            }

            @Override
            public void describeTo(Description description) {
                description.appendText("added = false, deleted = false");
            }
        };
    }

}
