/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.roster;

import com.kerio.im.bean.User;
import java.text.MessageFormat;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import org.hamcrest.BaseMatcher;
import org.hamcrest.Description;
import org.hamcrest.Matcher;
import tigase.server.Iq;
import tigase.server.Packet;
import tigase.xml.Element;
import tigase.xmpp.BareJID;
import tigase.xmpp.JID;
import tigase.xmpp.StanzaType;

/**
 *
 * @author tskoda
 */
public class PacketUtil {
    
    protected static final String EXPECTED = "<from=null, to={0}, DATA=<iq id=\"rst0\" to=\"{0}\" xmlns=\"jabber:client\" type=\"set\"><query xmlns=\"jabber:iq:roster\"><item name=\"{1}\" jid=\"{2}\" subscription=\"{3}\"><group>group</group></item></query></iq>, SIZE=219, XMLNS=jabber:client, PRIORITY=NORMAL, PERMISSION=NONE, TYPE=set>";

    PacketUtil() {
    }

    private List<String> groups(Element item) {
        List<String> result = new LinkedList<String>();
        List<Element> children = item.getChildren();
        if (children != null) {
            for (Element child : children) {
                if (child.getName().equals("group")) {
                    result.add(child.getCData().trim());
                }
            }
        }
        return result;
    }

    private boolean checkItem(List<Element> items, final BareJID who, final String subscription, final String name, final List<String> groups) {
        boolean result = false;
        for (Element item : items) {
            BareJID editedUser = BareJID.bareJIDInstanceNS(item.getAttribute("jid"));
            if (editedUser.equals(who)) {
                result = item.getAttribute("subscription").equals(subscription);
                if (result && name != null) {
                    result = item.getAttribute("name").equals(name);
                }
                if (result) {
                    result = groups(item).equals(groups);
                }
                break;
            }
        }
        return result;
    }

    protected Matcher<Element> element(final BareJID who, final String subscription, final String name, final List<String> groups) {
        return new BaseMatcher<Element>() {
            @Override
            public boolean matches(Object item) {
                Element toMatch = (Element) item;
                
                boolean result = BareJID.bareJIDInstanceNS(toMatch.getAttribute("jid")).equals(who);
                
                if (result) {
                    result = toMatch.getAttribute("subscription").equals(subscription);
                    if (result) {
                        result = toMatch.getAttribute("name").equals(name);
                        if (result) {
                            result = groups(toMatch).equals(groups);
                        }
                    }
                }
                return result;
            }

            @Override
            public void describeTo(Description description) {
                description.appendText(MessageFormat.format("<item name=\"{0}\" jid=\"{1}\" subscription=\"{2}\"><group>{3}</group></item>", name, who, subscription, groups));
            }
        };
    }

    protected Matcher<Packet> packet(final JID to, final BareJID who, final String subscription, final String name, final List<String> groups) {
        return new BaseMatcher<Packet>() {
            @Override
            public boolean matches(Object packetObj) {
                boolean result;
                Iq packet = (Iq) packetObj;
                result = (packet.getType() == StanzaType.set) && (packet.getTo().equals(to));
                if (result) {
                    List<Element> items = packet.getElemChildren("iq/query/");
                    result = items.size() == 1;
                    if (result) {
                        result = checkItem(items, who, subscription, name, groups);
                    }
                }
                return result;
            }

            @Override
            public void describeTo(Description description) {
                description.appendText(MessageFormat.format(EXPECTED, to, name, who, subscription));
            }
        };
    }

    //        protected Matcher<Packet> multipacket(final JID to, final String subscription, final User... whos) {
    //
    //            return new BaseMatcher<Packet>() {
    //                @Override
    //                public boolean matches(Object packetObj) {
    //                    boolean result;
    //
    //                    Iq packet = (Iq) packetObj;
    //
    //                    result = (packet.getType() == StanzaType.set) && (packet.getTo().equals(to));
    //                    if (result) {
    //                        List<Element> items = packet.getElemChildren("iq/query/");
    //                        result = items.size() == whos.length;
    //                        if (result) {
    //                            for (User user : whos) {
    //                                result = checkItem(items, as.bare(user), subscription, user.getFullName());
    //                                if (!result) {
    //                                    break;
    //                                }
    //                            }
    //                        }
    //                    }
    //
    //                    return result;
    //                }
    //
    //                @Override
    //                public void describeTo(Description description) {
    //                    description.appendText(MessageFormat.format(EXPECTED, to, name, who, subscription));
    //                }
    //            };
    //        }
    //<iq type="set" to="psi@tskoda-win.kerio.local/tskoda-win" id="rsttig3">
    //<query xmlns="jabber:iq:roster">
    //<item subscription="both" name="" jid="added@tskoda-win.kerio.local">
    //<group>Colleagues</group>
    //</item>
    //</query>
    //</iq>
    protected Matcher<Packet> added(User owner, int resource, User added, List<String> groups) {
        return packet(RosterNotifyTest.as.jid(RosterNotifyTest.as.bare(owner), resource), RosterNotifyTest.as.bare(added), "both", added.getFullName(), groups);
    }

    //        protected Matcher<Packet> added(User owner, int resource, User... added) {
    //            return multipacket(as.jid(as.bare(owner), resource), "both", added);
    //        }
    //<iq type="set" to="psi@tskoda-win.kerio.local/tskoda-win" id="rsttig2">
    //<query xmlns="jabber:iq:roster">
    //<item subscription="remove" jid="heslo@tskoda-win.kerio.local"/>
    //</query>
    //</iq>
    protected Matcher<Packet> removed(User owner, int resource, User removed) {
        return packet(RosterNotifyTest.as.jid(RosterNotifyTest.as.bare(owner), resource), RosterNotifyTest.as.bare(removed), "remove", null, Collections.EMPTY_LIST);
    }
    //<iq type="set" to="psi@tskoda-win.kerio.local/tskoda-win" id="rsttig4">
    //<query xmlns="jabber:iq:roster">
    //<item subscription="both" name="iChat Client renamed" jid="ichat@tskoda-win.kerio.local">
    //<group>Colleagues</group>
    //</item>
    //</query>
    //</iq>

    protected Matcher<Packet> renamed(User owner, int resource, User renamed, List<String> groups) {
        return packet(RosterNotifyTest.as.jid(RosterNotifyTest.as.bare(owner), resource), RosterNotifyTest.as.bare(renamed), "both", renamed.getFullName(), groups);
    }
    //<iq type="set" to="psi@tskoda-win.kerio.local/tskoda-win" id="rsttig3">
    //<query xmlns="jabber:iq:roster">
    //<item subscription="?" name="" jid="added@tskoda-win.kerio.local">
    //<group>Colleagues</group>
    //</item>
    //</query>
    //</iq>
    //        protected Matcher<Packet> visibled(User owner, int resource, User... roster) {
    //            return multipacket(as.jid(as.bare(owner), resource), RosterAbstract.SubscriptionType.both.toString(), roster);
    //        }
    //
    //        protected Matcher<Packet> invisibled(User owner, int resource, User... roster) {
    //            return multipacket(as.jid(as.bare(owner), resource), RosterAbstract.SubscriptionType.to.toString(), roster);
    //        }
}
