/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.user;

import org.hamcrest.BaseMatcher;
import org.hamcrest.Description;
import org.hamcrest.Matcher;
import tigase.xmpp.impl.roster.RosterAbstract;
import tigase.xmpp.impl.roster.RosterElement;

/**
 *
 * @author tskoda
 */
public class RosterElementUtil {

    public Matcher<RosterElement> subscribed(final RosterAbstract.SubscriptionType type) {
        return new BaseMatcher<RosterElement>() {
            @Override
            public boolean matches(Object item) {
                RosterElement toTest = (RosterElement) item;
                return toTest.getSubscription() == type;
            }

            @Override
            public void describeTo(Description description) {
                description.appendText("with subscription " + type.toString().toUpperCase());
            }
        };
    }

    public Matcher<RosterElement> has(final Matcher<RosterElement> matcher) {
        return new BaseMatcher<RosterElement>() {
            @Override
            public boolean matches(Object item) {
                return matcher.matches(item);
            }

            @Override
            public void describeTo(Description description) {
                matcher.describeTo(description);
            }
        };
    }
    
    public Matcher<RosterElement> has(final Matcher<RosterElement>... matchers) {
        return new BaseMatcher<RosterElement>() {
            @Override
            public boolean matches(Object item) {
                for (Matcher<RosterElement> m: matchers) {
                    if (! m.matches(item)) {
                        return false;
                    }
                }
                return true;
            }

            @Override
            public void describeTo(Description description) {
                for (Matcher<RosterElement> m: matchers) {
                    m.describeTo(description);
                }
            }
        };
    }

    public Matcher<RosterElement> group(final String group) {
        return new BaseMatcher<RosterElement>() {
            @Override
            public boolean matches(Object item) {
                RosterElement toTest = (RosterElement) item;
                if (group == null) {
                    return toTest.getGroups() == null || toTest.getGroups().length == 0;
                } else {
                    return toTest.getGroups().length == 1 && toTest.getGroups()[0].equals(group);
                }
                
            }

            @Override
            public void describeTo(Description description) {
                description.appendText("in group " + group);
            }
        };
    }

    public Matcher<RosterElement> name(final String name) {
        return new BaseMatcher<RosterElement>() {
            @Override
            public boolean matches(Object item) {
                RosterElement toTest = (RosterElement) item;
                return toTest.getName().equals(name);
            }

            @Override
            public void describeTo(Description description) {
                description.appendText("with name " + name);
            }
        };
    }
}
