/*
 *  Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.user;

import com.kerio.im.tigase.SessionProvider;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.mockito.Mockito;
import tigase.xmpp.BareJID;
import tigase.xmpp.NoConnectionIdException;
import tigase.xmpp.NotAuthorizedException;

/**
 *
 * @author tskoda
 */
public class TestSessionProvider extends SessionProvider {

    private static final UserUtil as = new UserUtil();

    private final Map<BareJID, Integer> online = new HashMap<BareJID, Integer>();
    private final Set<BareJID> disconnected = new HashSet<BareJID>();

    public TestSessionProvider() {
        super(null, null);
    }

    @Override
    public List<SessionProvider.UserResource> getUserResources(final BareJID jid) throws NotAuthorizedException {
        List<SessionProvider.UserResource> result;
        if (this.online.containsKey(jid)) {
            int count = this.online.get(jid);
            result = new ArrayList<SessionProvider.UserResource>(count);
            for (int i = 0; i < count; i++) {
                SessionProvider.UserResource rc = Mockito.mock(SessionProvider.UserResource.class);
                Mockito.when(rc.isAuthorized()).thenReturn(true);
                Mockito.when(rc.nextStanzaId()).thenReturn("" + i);
                Mockito.when(rc.getBareJID()).thenReturn(jid);
                Mockito.when(rc.getJID()).thenReturn(as.jid(jid, i));
                try {
                    Mockito.when(rc.getConnectionId()).thenReturn(as.jid(jid, i));
                } catch (NoConnectionIdException ex) {
                    ex.printStackTrace();
                }
                result.add(rc);
            }
        } else {
            result = Collections.EMPTY_LIST;
        }
        return result;
    }

    @Override
    public SessionProvider.UserData getUserData(BareJID jid) {
        return Mockito.mock(SessionProvider.UserData.class);
    }

    @Override
    public void disconnect(BareJID jid) throws NotAuthorizedException {
        this.disconnected.add(jid);
    }

    @Override
    public boolean isOnline(BareJID jid) throws NotAuthorizedException {
        return this.online.containsKey(jid);
    }

    public void online(int... onlineIds) {
        for (int id : onlineIds) {
            this.online.put(as.bare(id), 1);
        }
    }

    public void online(BareJID... onlineJids) {
        for (BareJID id : onlineJids) {
            this.online.put(id, 1);
        }
    }

    public void resources(int user, int resources) {
        this.online.put(as.bare(user), resources);
    }
    
    public void resources(BareJID user, int resources) {
        this.online.put(user, resources);
    }

    public Map<BareJID, Integer> getOnline() {
        return online;
    }

    public Set<BareJID> getDisconnected() {
        return disconnected;
    }

}
