/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.user;

import com.kerio.im.bean.User;
import java.sql.SQLException;
import java.util.Map;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;
import org.junit.Test;
import tigase.xmpp.BareJID;
import tigase.xmpp.impl.roster.RosterElement;

/**
 *
 * @author tskoda
 */
public class XmppUsersStoreAddTest extends XmppUsersStoreAbstract {

    @Test
    public void testAddOne() throws SQLException {
        this.store.add(as.user(1)); //enabled visible user
        
        assertTrue(database.all().size() == 1);
        assertTrue(database.roster.of(1).isEmpty());
        User savedUser = database.user(1);
        assertTrue(savedUser.isEnabled());
        assertTrue(savedUser.isVisible());
    }

    @Test
    public void testAddOneTwice() throws SQLException {
        this.store.add(as.user(1));
        
        assertFalse(this.store.add(as.user(1)));
        assertTrue(database.all().size() == 1);
        assertTrue(database.roster.of(1).isEmpty());
        User savedUser = database.user(1);
        assertTrue(savedUser.isEnabled());
        assertTrue(savedUser.isVisible());
    }

    @Test
    public void testAddDisabled() throws SQLException {
        this.store.add(as.disabled(1));
        
        assertTrue(database.all().size() == 1);
        assertTrue(database.roster.of(1).isEmpty());
        User savedUser = database.user(1);
        assertFalse(savedUser.isEnabled());
        assertTrue(savedUser.isVisible());
    }

    @Test
    public void testAddNonVisible() throws SQLException {
        this.store.add(as.invisible(1));

        assertTrue(database.all().size() == 1);
        assertTrue(database.roster.of(1).isEmpty());
        User savedUser = database.user(1);
        assertTrue(savedUser.isEnabled());
        assertFalse(savedUser.isVisible());
    }

    @Test
    public void testAddMultiple() throws SQLException {
        User[] users = {as.user(0), as.user(1), as.user(2)};
        this.store.add(users[0]);
        this.store.add(users[1]);
        this.store.add(users[2]);

        assertTrue(database.all().size() == 3);

        Map<BareJID, RosterElement> roster = database.roster.of(0);
        assertThat(roster.size(), is(0));

        roster = database.roster.of(1);
        assertThat(roster.size(), is(0));

        roster = database.roster.of(2);
        assertThat(roster.size(), is(0));
    }

    @Test
    public void testAddMultipleAllDisabled() throws SQLException {
        User[] users = {as.disabled(0), as.disabled(1), as.disabled(2)};
        this.store.add(users[0]);
        this.store.add(users[1]);
        this.store.add(users[2]);

        assertTrue(database.all().size() == 3);

        Map<BareJID, RosterElement> roster = database.roster.of(0);
        assertThat(roster.size(), is(0));

        roster = database.roster.of(1);
        assertThat(roster.size(), is(0));

        roster = database.roster.of(2);
        assertThat(roster.size(), is(0));
    }
}
