/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.user;

import com.kerio.im.bean.User;
import java.sql.SQLException;
import static org.junit.Assert.*;
import org.junit.Test;

/**
 *
 * @author tskoda
 */
public class XmppUsersStoreRemoveTest extends XmppUsersStoreAbstract {

    @Test
    public void testRemove() throws SQLException {
        User[] users = {as.user(0), as.user(1), as.user(2)};
        this.store.add(users[0]);
        this.store.add(users[1]);
        this.store.add(users[2]);

        assertTrue(database.all().size() == 3);

        repositoryReconnect();
        assertTrue(this.store.remove(as.user(1)));
        assertFalse(this.store.remove(as.user(1))); //user removed twice doesnt do anything
        assertFalse(this.store.remove(as.user(5)));

        assertTrue(database.roster.of(0).isEmpty());
        assertTrue(database.roster.of(1).isEmpty());
        assertTrue(database.roster.of(2).isEmpty());
    }

    @Test
    public void testRemoveDisabled() throws SQLException {
        User[] users = {as.user(0), as.disabled(1), as.user(2), as.disabled(3)};
        this.store.add(users[0]);
        this.store.add(users[1]);
        this.store.add(users[2]);
        this.store.add(users[3]);

        assertTrue(database.all().size() == 4);

        repositoryReconnect();

        assertTrue(this.store.remove(as.user(1)));

        assertTrue(database.roster.of(0).isEmpty());
        assertTrue(database.roster.of(1).isEmpty());
        assertTrue(database.roster.of(2).isEmpty());
    }

    @Test
    public void testRemoveInvisible() throws SQLException {
        User[] users = {as.user(0), as.invisible(1), as.user(2), as.invisible(3)};
        this.store.add(users[0]);
        this.store.add(users[1]);
        this.store.add(users[2]);
        this.store.add(users[3]);

        assertTrue(database.all().size() == 4);

        repositoryReconnect();

        assertTrue(this.store.remove(as.user(3)));

        assertTrue(database.roster.of(0).isEmpty());
        assertTrue(database.roster.of(1).isEmpty());
        assertTrue(database.roster.of(2).isEmpty());
    }
}
