/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.tigase.user;

import com.kerio.im.bean.User;
import com.kerio.im.xmpp.api.XmppUsers;
import java.sql.SQLException;
import java.util.EnumSet;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;
import org.junit.Test;
import tigase.xmpp.NotAuthorizedException;

/**
 *
 * @author tskoda
 */
public class XmppUsersStoreTest extends XmppUsersStoreAbstract {

    @Test
    public void testDisconnectOnRemove() throws NotAuthorizedException, SQLException {
        User[] users = {as.user(0), as.user(1), as.user(2)};
        repositoryAddAll(users);

        assertTrue(this.sessions.getDisconnected().isEmpty());

        //single
        repositoryReconnect();
        this.store.remove(as.user(0));
        assertThat(this.sessions.getDisconnected().size(), is(1));
        assertTrue(this.sessions.getDisconnected().contains(as.bare(0)));

        //multiple
        repositoryReconnect();
        this.store.remove(as.user(1));
        this.store.remove(as.user(2));
        
        assertThat(this.sessions.getDisconnected().size(), is(2));
        assertTrue(this.sessions.getDisconnected().contains(as.bare(1)));
        assertTrue(this.sessions.getDisconnected().contains(as.bare(2)));

        //non existent
        repositoryReconnect();
        this.store.remove(as.user(10));
        assertThat(this.sessions.getDisconnected().size(), is(1));
        assertTrue(this.sessions.getDisconnected().contains(as.bare(10)));
    }

    @Test
    public void testDisconnectOnUpdateStatus() throws NotAuthorizedException, SQLException {
        User[] users = {as.user(0), as.user(1), as.disabled(2)};
        repositoryAddAll(users);
        assertTrue(this.sessions.getDisconnected().isEmpty());

        repositoryReconnect(); //update with any change (enabled user)
        this.store.update(as.user(0), EnumSet.of(XmppUsers.Changes.STATUS));
        assertTrue(this.sessions.getDisconnected().isEmpty());

        repositoryReconnect(); //update to disabled
        this.store.update(as.disabled(1), EnumSet.of(XmppUsers.Changes.STATUS));
        assertThat(this.sessions.getDisconnected().size(), is(1));
        assertTrue(this.sessions.getDisconnected().contains(as.bare(1)));

        repositoryReconnect(); //update to enabled
        this.store.update(as.user(2), EnumSet.of(XmppUsers.Changes.STATUS));
        assertTrue(this.sessions.getDisconnected().isEmpty());

        repositoryReconnect(); //update to multiple changes
        this.store.update(as.disabled(2), EnumSet.of(XmppUsers.Changes.STATUS, XmppUsers.Changes.NAME, XmppUsers.Changes.VISIBILITY));
        assertThat(this.sessions.getDisconnected().size(), is(1));
        assertTrue(this.sessions.getDisconnected().contains(as.bare(2)));
    }

    @Test
    public void testDisconnectOnUpdate() throws NotAuthorizedException, SQLException {
        User[] users = {as.user(0), as.user(1), as.disabled(2)};
        repositoryAddAll(users);
        assertTrue(this.sessions.getDisconnected().isEmpty());

        repositoryReconnect();
        this.store.update(as.renamed(as.user(0), "new name user 0"), EnumSet.of(XmppUsers.Changes.NAME));
        assertTrue(this.sessions.getDisconnected().isEmpty());

        repositoryReconnect();
        this.store.update(as.invisible(0), EnumSet.of(XmppUsers.Changes.VISIBILITY));
        assertTrue(this.sessions.getDisconnected().isEmpty());
    }

    @Test
    public void testDisconnectOnAdd() throws NotAuthorizedException, SQLException {
        User[] users = {as.user(0), as.user(1), as.disabled(2)};
        repositoryAddAll(users);
        
        assertTrue(this.sessions.getDisconnected().isEmpty());

        repositoryReconnect();
        this.store.add(as.user(3));
        
        assertTrue(this.sessions.getDisconnected().isEmpty());

        repositoryReconnect();
        this.store.add(as.disabled(4));

        assertTrue(this.sessions.getDisconnected().isEmpty());

        repositoryReconnect();
        this.store.add(as.invisible(5));

        assertTrue(this.sessions.getDisconnected().isEmpty());
    }
}
