/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.xmpp.api;

import com.kerio.im.bean.User;
import java.util.Collection;
import java.util.EnumSet;
import java.util.Map;

/**
 * Interface provides methods for accessing users and their rosters inside xmpp
 * server. It manages users from single domain. All operations are 
 *
 * @author tskoda
 */
public interface XmppUsers {

    enum Changes {

	STATUS,
	NAME,
	VISIBILITY
    }

    /**
     * Check if user exists
     *
     * @param userName user to check, without domain part
     * @return true if user exist
     */
    boolean check(User user);

    /**
     * Returns all known users from domain it manages.
     *
     * @return
     */
    Map<String, User> all();

    /**
     * Add user to repository if not exist yet. Add user to all other users
     * rosters. To apply changes commit must be called.
     *
     * @param user user to add, never null
     * @see Rosters#commit()
     */
    boolean add(User user);

    /**
     * Remove user from database and in all other users rosters. To apply
     * changes commit must be called.
     *
     * @param user user to remove, never null
     * @see Rosters#commit()
     */
    boolean remove(User user);

    /**
     * Rename user in database and in all other users rosters. To apply changes
     * commit must be called.
     *
     * @param user user to rename, never null
     * @see Rosters#commit()
     */
    boolean update(User user, EnumSet<XmppUsers.Changes> changes);

    /**
     * Repair user roster. Renew connections between him and other users.
     * Convert unmanaged roster elements to managed. Move them to correct group.
     *
     * @param user user to repair, never null
     * @see Rosters#commit()
     */
    void repair(User user);

    void bye();
}
