/*
 * Copyright 2013 Kerio Technologies s.r.o. All rights reserved.
 */
package com.kerio.im.xmpp.api.event;

import com.kerio.im.core.api.service.ServiceBus;

/**
 *
 * @author tskoda
 */
public class PingEvent implements ServiceBus.Event {
    
    private static final String PING_ID = "ping_";
    
    public static PingEvent response(Type type, PingEvent request) {
        return new PingEvent(type, request);
    }
    
    public static String getPingId(long id) {
        return PING_ID + id;
    }
    
    public static boolean isPingId(String id) {
        return id.startsWith(PING_ID);
    }
    
    public enum Type {
        REQUEST,
        RESPONSE_SUCCESS,
        RESPONSE_FAILED
    }
    
    private final Type type;
    private final String sender;
    private final String recipient;
    private final String id;

    public PingEvent(Type type, String id, String sender, String recipient) {
        this.type = type;
        this.sender = sender;
        this.recipient = recipient;
        this.id = id;
    }
    
    private PingEvent(Type type, PingEvent event) {
        this.type = type;
        this.sender = event.sender;
        this.recipient = event.recipient;
        this.id = event.id;
    }

    public Type getType() {
        return type;
    }

    public String getSender() {
        return sender;
    }

    public String getRecipient() {
        return recipient;
    }

    public String getId() {
        return id;
    }
    
}
