/*
 * Copyright 2010 Kerio Technologies s.r.o. All rights reserved
 */
package tigase.io;

import java.util.concurrent.atomic.AtomicBoolean;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author tskoda
 */
public class SSLFilterProvider {

    private static final Logger LOG = Logger.getLogger(SSLFilterProvider.class.getName());
    private static final AtomicBoolean CIPHERS_WARNING_LOGGED = new AtomicBoolean(false);
    private static final AtomicBoolean PROTOCOLS_WARNING_LOGGED = new AtomicBoolean(false);

    private static SSLFilter INSTANCE;

    //this method is by design called in early stage of server initialization, anyway before first attempt to open any connection
    public static void setSSLFilter(SSLFilter instance) {
        INSTANCE = instance;
    }

    public static String[] filterCiphers(String protocol, String[] defaultEnabledCiphers) {
        if (INSTANCE != null) {
            return INSTANCE.filterCiphers(protocol, defaultEnabledCiphers);
        } else {
            //this could be caused only by incomplete refactoring
            if (! CIPHERS_WARNING_LOGGED.getAndSet(true)) {
                LOG.log(Level.FINE, "No ciphers available for SSL connections, cipher filter is not provided.");
                LOG.log(Level.WARNING, "No ciphers available for SSL connections. All secure connections to Instant messaging service are going to be rejected.");
            }

            return new String[0];
        }
    }
    
    public static String[] filterProtocols(String[] defaultEnabledProtocols) {
        if (INSTANCE != null) {
            return INSTANCE.filterProtocols(defaultEnabledProtocols);
        } else {
            //this could be caused only by incomplete refactoring
            if (! PROTOCOLS_WARNING_LOGGED.getAndSet(true)) {
                LOG.log(Level.FINE, "No protocols available for SSL connections, protocol filter is not provided.");
                LOG.log(Level.WARNING, "No protovols available for SSL connections. All secure connections to Instant messaging service are going to be rejected.");
            }

            return new String[0];
        }
    }
}
